<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Paul Oehler <poehler@interworx.com>
 * @date       Nov 18, 2008
 */

/**
 * Used as a base class for command line based scripts.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
abstract class CLITestCase extends PHPUnit_Framework_TestCase {

  const QUICK_TEST_TIME_LIMIT = 15;

  /**
   * Gets the name of the command line script we're testing.
   */
  abstract protected function _getScriptToTest();

  /**
   * Gets the command line parameters for testing.  Is a dataProvider.
   *
   * @return array
   */
  abstract public function getCommandlineTestInfo();

  /**
   * @var integer.
   */
  protected $_maxRunningTime = 0;

  static protected $_script_variables = [];

  /**
   * Adds a variable to the replacements used before execution.
   *
   * @param string $name
   * @param mixed  $value
   */
  static public function addVariable( $name, $value ) {
    self::$_script_variables["%%{$name}%%"] = $value;
  }

  /**
   * Run Test.
   *
   * @return mixed
   */
  public function runTest() {
    PHP_Timer::start();
    $testResult = parent::runTest();
    $time       = PHP_Timer::stop();

    if( $this->_maxRunningTime != 0 &&
        $time > $this->_maxRunningTime ) {
      $this->fail( sprintf( 'expected running time: <= %s but was: %s',
                            $this->_maxRunningTime,
                            $time
                            ) );
    }

    return $testResult;
  }

  /**
   * Set Max Running Time.
   *
   * @param integer $maxRunningTime
   */
  public function setMaxRunningTime( $maxRunningTime ) {
    if( is_integer( $maxRunningTime ) &&
        $maxRunningTime >= 0 ) {
      $this->_maxRunningTime = $maxRunningTime;
    } else {
      throw PHPUnit_Util_InvalidArgumentHelper::factory( 1, 'positive integer' );
    }
  }

  /**
   * Get max running time.
   *
   * @return integer
   * @since  Method available since Release 2.3.0
   */
  public function getMaxRunningTime() {
    return $this->_maxRunningTime;
  }

  /**
   * Setup.
   */
  public function setUp() {
    $this->_setMaxTimeLimit();
  }

  /**
   * Sets the max time limit depending on the group selection.
   */
  private function _setMaxTimeLimit() {
    $opts = array( 'exclude-group=s' => '' );
    $Opt  = new IWorx_Console_Getopt( $opts );
    if( $Opt->getOption( 'exclude-group' ) === 'slow-tests' ) {
      $this->setMaxRunningTime( self::QUICK_TEST_TIME_LIMIT );
    } else {
      $this->setMaxRunningTime( 0 );
    }
  }

  /**
   * Test command line script.
   *
   * @param array  $parameters
   * @param string $expected_output
   * @param string $expected_retval
   * @param string $callback_func
   * @param array  $callback_params
   *
   * @dataProvider getCommandlineTestInfo
   */
  public function testCommandlineScript( array $parameters,
                                         $expected_output,
                                         $expected_retval,
                                         $callback_func = null,
                                         array $callback_params = [] ) {

    $bin = $this->_getScriptToTest();

    if( $bin === Ini::get( Ini::APF_BIN, 'apf' ) && Node::isInCluster() ) {
      $msg = 'This node is in a cluster, we don\'t want to be messing ' .
        'around with the firewall. ;)';
      $this->markTestSkipped( $msg );
    }

    $cmd = "$bin " . implode( ' ', $parameters );

    $cmd = strtr( $cmd, self::$_script_variables );

    IWorxExec::exec( $cmd, $result, $retval, IWorxExec::STDERR_2_STDOUT );

    if( $expected_output !== implode( "\n", $result ) ) {
      echo "\n\n";
      echo implode( "\n", $result );
      echo "\nRetval: {$retval}\n";
    }

    $this->assertEquals( $expected_output,
                         implode( "\n", $result ),
                         'output did not match expected output: ' . $cmd );
    $this->assertEquals( $expected_retval,
                         $retval,
                         'return value did not match expected return value' );

    if( !is_null( $callback_func ) ) {
      call_user_func_array( array( $this, $callback_func ), $callback_params );
    }
  }

}


