<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Tim College <tcollege@interworx.com>
 * @date    Nov 3, 2009
 */

/**
 * Tests for SimpleScripts plugin.
 *
 * @package    InterWorx
 * @subpackage Tests
 * @group      selenium
 */
class PageNWSimpleScriptsTest extends IWorxSeleniumTestCase {
  const SS_EMAIL = 'simplescripts@testing.com';

  private $_sw_menu_el = "//a[@id='menu-simplescripts']";
  private $_sw_perm_el = "//table[@id='perms_table']//td[label='SimpleScripts']";

  /**
   * Performs a few pre-test actions.
   */
  public function setUp() {
    parent::setUp();
    Plugin_Manager::clearInstance();
    Form::resetInstances();
  }

  /**
   * Tests that the plugin is initially disabled.
   */
  public function testInitallyEnabled() {
    $this->deleteSWAccountIfNecessary();
    $this->addSWAccountIfNecessary();
    IW::PluginManager()->getPlugin( 'simplescripts' )->deleteFromDatastore();
    $is_enabled = IW::PluginManager()->getPlugin( 'simplescripts' )->isEnabled();
    if( $is_enabled === false ) {
      Ini::set( Ini::PLUGINS, 'simplescripts', '1' );
      Ini::write();
    }
  }

  /**
   * Verifies that elements are present in NodeWorx.
   */
  public function testNWElementsPresent() {
    $this->loginToNWAndGoTo( 'nodeworx/siteworx?action=add&reseller=1' );

    $this->assertElementPresent( $this->_sw_menu_el,
                                 'Menu item was not present while enabled' );

    $this->assertElementPresent( "//*[@id='simplescripts']",
                                 'SiteWorx Option not present while enabled' );

    $this->addSWAccountIfNecessary();
    $blocked = IW::PluginManager()->getPlugin( 'simplescripts' )
      ->isAccountBlocked( 'addedbyt' );

    $this->assertFalse( $blocked, 'ABT account was blocked' );

    $this->loginToNWAndGoTo( 'nodeworx/simplescripts' );

    $this->assertTextPresent( 'Host and API Key', 'The form appears to be missing' );
  }

  /**
   * Verifies that elements are present in SiteWorx.
   */
  public function testSWElementsPresent() {
    $this->loginToSWAndGoTo( 'siteworx/users?action=add' );

    $this->assertElementPresent( $this->_sw_menu_el,
                                 'Menu item was not present while enabled' );

    $this->assertElementPresent( $this->_sw_perm_el,
                                 'Permission was not available while enabled' );
  }

  /**
   * Attempts to disable SimpleScripts.
   */
  public function testDisableSimpleScripts() {
    $str = 'nodeworx/plugins?action=edit&plugin_name=simplescripts';
    $this->loginToNWAndGoTo( $str );
    $this->select( 'status', 'value=0' );
    $this->clickSave();
    $this->waitForMessageBanner();
    $this->assertTrue( $this->isTextPresent( 'Plugin Saved - ' ) );

    $this->waitForPageToLoad();

    IW::PluginManager()->clearInstance();
    Ini::reload();
    $is_enabled = IW::PluginManager()->getPlugin( 'simplescripts' )->isEnabled();
    $this->assertFalse( $is_enabled, 'Simplescripts not disabled!' );
  }

  /**
   * Verifies that elements are not present in NodeWorx.
   */
  public function testNWElementsNotPresent() {
    $this->loginToNWAndGoTo( 'nodeworx/siteworx?action=add&reseller=1' );

    // Verify that the menu item is not there
    $this->assertElementNotPresent( $this->_sw_menu_el,
                                    'Menu item was present while disabled' );

    // Verify that the SiteWorx Option is not there
    $this->assertElementNotPresent( "//*[@id='simplescripts']",
                                    'SiteWorx Option present while disabled' );

    $this->loginToNWAndGoTo( 'nodeworx/simplescripts?action=launch' );

    // Verify that accessing the controller fails
    $this->assertTextPresent( "Unknown Controller: 'Ctrl_Nodeworx_Simplescripts'",
                              'No error msg when trying to access SimpleScripts' );
  }

  /**
   * Verifies that SiteWorx elements are unavailable.
   */
  public function testSWElementsNotPresent() {
    $this->loginToSWAndGoTo( 'siteworx/users?action=add' );

    $this->assertElementNotPresent( $this->_sw_menu_el,
                                    'Menu item was not present while enabled' );

    $message = 'Permission was not available to account with option';
    $this->assertElementNotPresent( $this->_sw_perm_el, $message );
  }

  /**
   * Attempts to enable SimpleScripts.
   */
  public function testEnableSimpleScripts() {
    $str = 'nodeworx/plugins?action=edit&plugin_name=simplescripts';
    $this->loginToNWAndGoTo( $str );
    $this->select( 'status', 'value=1' );
    $this->clickSave();
    $this->waitForMessageBanner();
    $this->assertTrue( $this->isTextPresent( 'Plugin Saved - ' ) );

    $this->waitForPageToLoad();
    IW::PluginManager()->clearInstance();
    Ini::reload();
    $is_enabled = IW::PluginManager()->getPlugin( 'simplescripts' )->isEnabled();
    $this->assertTrue( $is_enabled, 'Simplescripts enable failed.' );
  }

  /**
   * Tests a secondary user that doesn't have permission for SimpleScripts.
   */
  public function testSecondaryUserWithoutPermission() {
    if( $this->isSWSecondaryUserAlreadyCreated( self::SS_EMAIL ) ) {
      $SW    = new SiteWorx( self::SW_DOMAIN );
      $Reply = IW::FC()->routeFromPHP( 'Ctrl_Siteworx_Users',
                                       'delete',
                                       array( 'user' => self::SS_EMAIL ),
                                       $SW );
    }
    // Adds a SW user with all the "base" permissions, but no plugin perms
    $this->addSWSecondaryUserIfNecessary( self::SS_EMAIL );

    $this->loginToSWAndGoTo( 'siteworx/users?action=add',
                             self::SS_EMAIL,
                             self::SW_SECONDARY_PASS );

    $this->assertElementNotPresent( $this->_sw_menu_el,
                                    'Menu item was available to user without perm' );

    $this->assertElementNotPresent( $this->_sw_perm_el,
                                    'Permission was available to user without it' );

    $this->loginToSWAndGoTo( 'siteworx/simplescripts' );

    $this->assertTextPresent( 'Access to SimpleScripts is blocked' );
  }

  /**
   * Tests that accounts without access can't see SimpleScripts.
   */
  public function testDisabledAccessToSimpleScripts() {
    $input = array( 'domain' => self::SW_DOMAIN,
                    'simplescripts' => '0' );
    $NW    = IWorx_Framework_TestCase::getMasterNodeWorx();

    Plugin_Manager::clearInstance();
    Ini::reload();

    $ctrl  = 'Ctrl_Nodeworx_Siteworx';
    $Reply =  IW::FC()->routeFromPHP( $ctrl, 'edit', $input, $NW );

    $this->assertReplySuccessful( $Reply );

    $this->loginToSWAndGoTo( 'siteworx/users?action=add' );

    $message = 'Menu item was available to account without option';
    $this->assertElementNotPresent( $this->_sw_menu_el, $message );

    $message = 'Permission was available to account without option';
    $this->assertElementNotPresent( $this->_sw_perm_el, $message );

    $this->loginToSWAndGoTo( 'siteworx/simplescripts' );

    $this->assertTextPresent( 'Access to SimpleScripts is blocked' );
  }

  /**
   * Tests that access comes back when permission is granted.
   */
  public function testRestoreSimpleScriptsAccess() {
    $input = array( 'domain' => self::SW_DOMAIN,
                    'simplescripts' => '1' );
    $NW    = IWorx_Framework_TestCase::getMasterNodeWorx();

    $ctrl  = 'Ctrl_Nodeworx_Siteworx';
    $Reply =  IW::FC()->routeFromPHP( $ctrl, 'edit', $input, $NW );
    $this->assertReplySuccessful( $Reply );

    Plugin_Manager::clearInstance();
    Ini::reload();

    $this->loginToSWAndGoTo( 'siteworx/users?action=add' );

    $message = 'Menu item was not available to account with option';
    $this->assertElementPresent( $this->_sw_menu_el, $message );

    $message = 'Permission was not available to account with option';
    $this->assertElementPresent( $this->_sw_perm_el, $message );

    $this->open( 'siteworx/simplescripts' );
    $this->waitForPageToLoad();

    $this->assertTextNotPresent( 'Access to SimpleScripts is blocked' );
  }
}
