<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Tim College <tcollege@interworx.com>
 * @date    Nov 3, 2009
 */

/**
 * Tests for Softaculous plugin.
 *
 * @package    InterWorx
 * @subpackage Tests
 * @group      selenium
 * @retryLimit 2
 */
class PageNWSoftaculousTest extends IWorxSeleniumTestCase {

  const TEST_EMAIL = 'softaculous@testing.com';

  /**
   * Xpath definition to find the permission in the SiteWorx User Add form.
   *
   * @var string
   */
  private $_sw_perm_element = "//table[@id='perms_table']//td[label='Softaculous']";

  /**
   * XPath definition to find the menu item.
   *
   * @var string
   */
  private $_sw_menu_element = "//a[@id='menu-softaculous']";

  /**
   * Performs a few pre-test actions.
   */
  public function setUp() {
    parent::setUp();
    Plugin_Manager::clearInstance();
    Form::resetInstances();
    Ini::reload();
  }

  /**
   * Tests that the plugin is initially disabled.
   */
  public function testInitallyDisabled() {
    $this->deleteSWAccountIfNecessary();
    $this->addSWAccountIfNecessary();
    IW::PluginManager()->getPlugin( 'softaculous' )->deleteFromDatastore();
    $is_enabled = IW::PluginManager()->getPlugin( 'softaculous' )->isEnabled();
    if( $is_enabled ) {
      Ini::set( Ini::PLUGINS, 'softaculous', '0' );
      Ini::write();
    }
  }

  /**
   * Verifies that elements are not present in NodeWorx.
   */
  public function testNWElementsNotPresent() {
    $this->loginToNWAndGoTo( 'nodeworx/siteworx?action=add&reseller=1' );

    // Verify that the menu item is not there
    $this->assertElementNotPresent( $this->_sw_menu_element,
                                    'Menu item was present while disabled' );

    // Verify that the SiteWorx Option is not there
    $this->assertElementNotPresent( "//*[@id='softaculous']",
                                    'SiteWorx Option present while disabled' );

    $this->loginToNWAndGoTo( 'nodeworx/softaculous?action=launch' );

    // Verify that accessing the controller fails
    $this->assertTextPresent( "Unknown Controller: 'Ctrl_Nodeworx_Softaculous'",
                              'No error message when trying to access Softaculous' );
  }

  /**
   * Verifies that SiteWorx elements are unavailable.
   */
  public function testSWElementsNotPresent() {
    $this->loginToSWAndGoTo( 'siteworx/users?action=add' );

    $this->assertElementNotPresent( $this->_sw_menu_element,
                                    'Menu item was not present while enabled' );

    $message = 'Permission was not available to account with option';
    $this->assertElementNotPresent( $this->_sw_perm_element, $message );
  }

  /**
   * Attempts to enable Softaculous.
   */
  public function testEnableSoftaculous() {
    $str = 'nodeworx/plugins?action=edit&plugin_name=softaculous';
    $this->loginToNWAndGoTo( $str );
    $this->select( 'status', 'value=1' );
    $this->clickSave();
    $this->waitForMessageBanner();
    $this->assertTrue( $this->isTextPresent( 'Plugin Saved - ' ) );

    $this->waitForPageToLoad();
    IW::PluginManager()->clearInstance();
    Ini::reload();
    $is_enabled = IW::PluginManager()->getPlugin( 'softaculous' )->isEnabled();
    $this->assertTrue( $is_enabled, 'Softaculous enable failed.' );
  }

  /**
   * Verifies that elements are present in NodeWorx.
   */
  public function testNWElementsPresent() {
    $this->loginToNWAndGoTo( 'nodeworx/siteworx?action=add&reseller=1' );

    $this->assertElementPresent( $this->_sw_menu_element,
                                 'Menu item was not present while enabled' );

    $this->assertElementPresent( "//*[@id='softaculous']",
                                 'SiteWorx Option not present while enabled' );

    $this->addSWAccountIfNecessary();
    $blocked = IW::PluginManager()->getPlugin( 'softaculous' )
      ->isAccountBlocked( 'addedbyt' );

    $this->assertFalse( $blocked, 'ABT account was blocked' );

    $this->loginToNWAndGoTo( 'nodeworx/softaculous?action=launch' );

    $this->assertElementPresent( "//iframe[@id='internal']",
                                 'Iframe is missing on Softaculous page' );
  }

  /**
   * Verifies that elements are present in SiteWorx.
   */
  public function testSWElementsPresent() {
    $this->loginToSWAndGoTo( 'siteworx/users?action=add' );

    $this->assertElementPresent( $this->_sw_menu_element,
                                 'Menu item was not present while enabled' );

    $element = $this->_sw_perm_element;
    $message = 'Permission was not available while enabled';
    $this->assertElementPresent( $element, $message );
  }

  /**
   * Tests a secondary user that doesn't have permission for Softaculous.
   */
  public function testSecondaryUserWithoutPermission() {
    if( $this->isSWSecondaryUserAlreadyCreated( self::TEST_EMAIL ) ) {
      $SW    = new SiteWorx( self::SW_DOMAIN );
      $Reply = IW::FC()->routeFromPHP( 'Ctrl_Siteworx_Users',
                                       'delete',
                                       array( 'user' => self::TEST_EMAIL ),
                                       $SW );
    }
    // Adds a SW user with all the "base" permissions, but no plugin perms
    $this->addSWSecondaryUserIfNecessary( self::TEST_EMAIL );

    $this->loginToSWAndGoTo( 'siteworx/users?action=add',
                             self::TEST_EMAIL,
                             self::SW_SECONDARY_PASS );

    $this->assertElementNotPresent( $this->_sw_menu_element,
                                    'Menu item was available to user without perm' );

    $this->assertElementNotPresent( $this->_sw_perm_element,
                                    'Permission was available to user without it' );

    $this->loginToSWAndGoTo( 'siteworx/softaculous' );

    $this->assertTextPresent( 'Access to Softaculous is blocked' );
  }

  /**
   * Tests that accounts without access can't see Softaculous.
   */
  public function testDisabledAccessToSoftaculous() {
    $input = array( 'domain' => self::SW_DOMAIN,
                    'softaculous' => '0' );
    $NW    = IWorx_Framework_TestCase::getMasterNodeWorx();
    $ctrl  = 'Ctrl_Nodeworx_Siteworx';
    $Reply = IW::FC()->routeFromPHP( $ctrl, 'edit', $input, $NW );

    $this->assertReplySuccessful( $Reply );

    $this->loginToSWAndGoTo( 'siteworx/users?action=add' );

    $message = 'Menu item was available to account without option';

    $this->assertElementNotPresent( $this->_sw_menu_element, $message );

    $message = 'Permission was available to account without option';

    $this->assertElementNotPresent( $this->_sw_perm_element, $message );

    // Gotta use open(), because loginToSW... waits for the menu
    $this->open( 'siteworx/softaculous' );
    $this->waitForPageToLoad();

    $this->assertTextPresent( 'Access to Softaculous is blocked' );
  }

  /**
   * Tests that access is restored when re-enabling access.
   */
  public function testRestoreSoftaculousAccess() {
    $input = array( 'domain' => self::SW_DOMAIN,
                    'softaculous' => '1' );
    $NW    = IWorx_Framework_TestCase::getMasterNodeWorx();
    $ctrl  = 'Ctrl_Nodeworx_Siteworx';
    $Reply = IW::FC()->routeFromPHP( $ctrl, 'edit', $input, $NW );
    $this->assertReplySuccessful( $Reply );

    $this->loginToSWAndGoTo( 'siteworx/users?action=add' );

    $message = 'Menu item was not available to account with option';

    $this->assertElementPresent( $this->_sw_menu_element, $message );

    $message = 'Permission was not available to account with option';

    $this->assertElementPresent( $this->_sw_perm_element, $message );

    // Gotta use open(), because loginToSW... waits for the menu
    $this->open( 'siteworx/softaculous' );
    $this->waitForPageToLoad();

    $this->assertTextNotPresent( 'Access to Softaculous is blocked' );
  }

  /**
   * Attempts to disable Softaculous.
   */
  public function testDisableSoftaculous() {
    $str = 'nodeworx/plugins?action=edit&plugin_name=softaculous';
    $this->loginToNWAndGoTo( $str );
    $this->select( 'status', 'value=0' );
    $this->clickSave();
    $this->waitForMessageBanner();
    $this->assertTrue( $this->isTextPresent( 'Plugin Saved - ' ) );

    $this->waitForPageToLoad();

    IW::PluginManager()->clearInstance();
    Ini::reload();
    $is_enabled = IW::PluginManager()->getPlugin( 'softaculous' )->isEnabled();
    $this->assertFalse( $is_enabled, 'Softaculous not disabled!' );
  }
}
