<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Paul Oehler <poehler@interworx.com>
 * @date    Jun 13, 2009
 */

/**
 * XMLRPC API Tests.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class API_XMLRPCTest extends IWorx_API_Framework_TestCase {

  /**
   * Test.
   */
  public function testNoParams() {
    $params = array( 'apikey'    => '',
                     'ctrl_name' => '',
                     'action'    => '',
                     'input'     => array() );

    $result = $this->_apiCall( 'route', $params );
    $this->assertEquals( 'Authentication Error (ApiKey)', $result['payload'] );
  }

  /**
   * Test.
   */
  public function testNodeworxWithApiKey() {
    $input  = [];
    $params = array( 'apikey'    => $this->_getGoodApiKey(),
                     'ctrl_name' => '/nodeworx/siteworx',
                     'action'    => 'winCommit',
                     'input'     => $input );

    $result = $this->_apiCall( 'route', $params );
    $this->_assertResponseIsWinBundle( $result );
  }

  /**
   * Test.
   */
  public function testNodeworxWithApiKeyWithNoHeader() {
    $key = $this->_getGoodApiKey();

    $keylines = explode( "\n", $key );
    unset( $keylines[ 0 ] );
    unset( $keylines[ count( $keylines ) ] );

    $key = implode( "\n", $keylines );

    $input  = [];
    $params = array( 'apikey'    => $key,
                     'ctrl_name' => '/nodeworx/siteworx',
                     'action'    => 'winCommit',
                     'input'     => $input );

    $result = $this->_apiCall( 'route', $params );
    $this->_assertResponseIsWinBundle( $result );
  }

  /**
   * Test (whmcs does this).
   */
  public function testNodeworxWithApiKeyWithNoHeaderNoNewlines() {
    $key = $this->_getGoodApiKey();

    $keylines = explode( "\n", $key );
    unset( $keylines[ 0 ] );
    unset( $keylines[ count( $keylines ) ] );

    $key = implode( '', $keylines );

    $input  = [];
    $params = array( 'apikey'    => $key,
                     'ctrl_name' => '/nodeworx/siteworx',
                     'action'    => 'winCommit',
                     'input'     => $input );

    $result = $this->_apiCall( 'route', $params );
    $this->_assertResponseIsWinBundle( $result );
  }

  /**
   * Test.
   */
  public function testNodeworxWithBadApiKey() {
    $input  = [];
    $params = array( 'apikey'    => 'hacker123',
                     'ctrl_name' => '/nodeworx/siteworx',
                     'action'    => 'winCommit',
                     'input'     => $input );

    $result = $this->_apiCall( 'route', $params );
    $this->assertEquals( 'Authentication Error (ApiKey)', $result['payload'] );
  }

  /**
   * Test.
   */
  public function testNodeworxGetSessionWithApiKey() {
    $input = [];

    $params = array( 'apikey'    => $this->_getGoodApiKey(),
                     'ctrl_name' => '/nodeworx/index',
                     'action'    => 'getSession',
                     'input'     => $input );

    $result = $this->_apiCall( 'route', $params );
    $this->_assertResponseLooksValid( $result );

    $session = $result['payload'];

    $this->assertFalse( empty( $session ) );

    $key    = array( 'sessionid' => $session );
    $params = array( 'apikey'    => $key,
                     'ctrl_name' => '/nodeworx/siteworx',
                     'action'    => 'winCommit',
                     'input'     => $input );

    $result = $this->_apiCall( 'route', $params );
    $this->_assertResponseIsWinBundle( $result );
  }

  /**
   * Tests logging in to SiteWorx using a NodeWorx API key.
   */
  public function testSiteworxWinWithApiKey() {
    self::addSWAccountIfNecessary();
    $key = array( 'apikey' => $this->_getGoodApiKey(),
                  'domain' => self::SW_DOMAIN );

    $params = array( 'apikey'    => $key,
                     'ctrl_name' => '/siteworx/overview',
                     'action'    => 'winCommit',
                     'input'     => array() );

    $result = $this->_apiCall( 'route', $params );
    $this->_assertResponseIsWinBundle( $result );
  }

  /**
   * Makes sure that logging in with a BS domain will fail.
   */
  public function testSiteworxFailWithApiKeyAndBadDomain() {
    $key = array( 'apikey' => $this->_getGoodApiKey(),
                  'domain' => 'Bullshizzle.com' );

    $params = array( 'apikey'    => $key,
                     'ctrl_name' => '/siteworx/overview',
                     'action'    => 'winCommit',
                     'input'     => array() );

    $result = $this->_apiCall( 'route', $params );

    $this->assertEquals( API::GENERAL_FAILURE, $result['status'] );
  }

  /**
   * Tests using siteworx actions after logging in with a NodeWorx API key.
   */
  public function testSiteworxActionsWithSiteworxLoginFunction() {
    $input = [];

    $params = array( 'apikey'    => $this->_getGoodApiKey(),
                     'ctrl_name' => '/nodeworx/index',
                     'action'    => 'getSession',
                     'input'     => $input );

    $result = $this->_apiCall( 'route', $params );

    $this->_assertResponseLooksValid( $result );

    $session = $result['payload'];

    $this->assertFalse( empty( $session ) );

    $key    = array( 'sessionid'      => $session );
    $input  = array( 'login_domain'   => self::SW_DOMAIN,
                     'account_action' => 'siteworxLogin' );

    $params = array( 'apikey'    => $key,
                     'ctrl_name' => '/nodeworx/siteworx',
                     'action'    => 'siteworxLogin',
                     'input'     => $input );

    $result = $this->_apiCall( 'route', $params );

    $this->_assertResponseLooksValid( $result );

    $input  = [];
    $params = array( 'apikey'    => $key,
                     'ctrl_name' => '/siteworx/email/box',
                     'action'    => 'winCommit',
                     'input'     => $input );

    $result = $this->_apiCall( 'route', $params );

    $this->_assertResponseIsWinBundle( $result );
  }

  /**
   * Test.
   */
  public function testNodeworxGetSessionWithLogin() {
    $input = [];
    $key   = array( 'email'    => self::NW_USER,
                    'password' => self::NW_PASS );

    $params = array( 'apikey'    => $key,
                     'ctrl_name' => '/nodeworx/index',
                     'action'    => 'getSession',
                     'input'     => $input );

    $result = $this->_apiCall( 'route', $params );
    $this->_assertResponseLooksValid( $result );

    $session = $result['payload'];

    $this->assertFalse( empty( $session ) );

    $key    = array( 'sessionid' => $session );
    $params = array( 'apikey'    => $key,
                     'ctrl_name' => '/nodeworx/siteworx',
                     'action'    => 'winCommit',
                     'input'     => $input );

    $result = $this->_apiCall( 'route', $params );
    $this->_assertResponseIsWinBundle( $result );
  }

  /**
   * Test.
   */
  public function testNodeworxSessionIdOnSiteworxController() {
    $input = [];
    $key   = array( 'email'    => self::NW_USER,
                    'password' => self::NW_PASS );

    $params = array( 'apikey'    => $key,
                     'ctrl_name' => '/nodeworx/index',
                     'action'    => 'getSession',
                     'input'     => $input );

    $result = $this->_apiCall( 'route', $params );
    $this->_assertResponseLooksValid( $result );

    $session = $result['payload'];

    $this->assertFalse( empty( $session ) );

    $key    = array( 'sessionid' => $session );
    $params = array( 'apikey'    => $key,
                     'ctrl_name' => '/siteworx/email/box',
                     'action'    => 'winCommit',
                     'input'     => $input );

    $result = $this->_apiCall( 'route', $params );
    $this->assertEquals( 'Authentication Error (SiteWorx)', $result['payload'] );
  }

  /**
   * Test.
   */
  public function testSiteworxGetSessionWithLogin() {
    $input = [];
    $key   = array( 'email'    => self::SW_EMAIL,
                    'password' => self::SW_PASS,
                    'domain'   => self::SW_DOMAIN );

    $params = array( 'apikey'    => $key,
                     'ctrl_name' => '/siteworx/index',
                     'action'    => 'getSession',
                     'input'     => $input );

    $result = $this->_apiCall( 'route', $params );
    $this->_assertResponseLooksValid( $result );

    $session = $result['payload'];

    $this->assertFalse( empty( $session ) );

    $key    = array( 'sessionid' => $session );
    $params = array( 'apikey'    => $key,
                     'ctrl_name' => '/siteworx/email/box',
                     'action'    => 'winCommit',
                     'input'     => $input );

    $result = $this->_apiCall( 'route', $params );
    $this->_assertResponseIsWinBundle( $result );
  }

  /**
   * Test.
   */
  public function testSiteworxSessionIdOnNodeworxController() {
    $input = [];
    $key   = array( 'email'    => self::SW_EMAIL,
                    'password' => self::SW_PASS,
                    'domain'   => self::SW_DOMAIN );

    $params = array( 'apikey'    => $key,
                     'ctrl_name' => '/siteworx/index',
                     'action'    => 'getSession',
                     'input'     => $input );

    $result = $this->_apiCall( 'route', $params );
    $this->_assertResponseLooksValid( $result );

    $session = $result['payload'];

    $this->assertFalse( empty( $session ) );

    $key    = array( 'sessionid' => $session );
    $params = array( 'apikey'    => $key,
                     'ctrl_name' => '/nodeworx/siteworx',
                     'action'    => 'winCommit',
                     'input'     => $input );

    $result = $this->_apiCall( 'route', $params );
    $this->assertEquals( 'Authentication Error (NodeWorx)', $result['payload'] );
  }

  /**
   * Test.
   */
  public function testNodeworxWithLogin() {
    $input = [];
    $key   = array( 'email'    => self::NW_USER,
                    'password' => self::NW_PASS );

    $params = array( 'apikey'    => $key,
                     'ctrl_name' => '/nodeworx/siteworx',
                     'action'    => 'winCommit',
                     'input'     => $input );

    $result = $this->_apiCall( 'route', $params );
    $this->_assertResponseIsWinBundle( $result );
  }

  /**
   * Test.
   */
  public function testNodeworxWithBadLogin() {
    $input = [];
    $key   = array( 'email'    => self::NW_USER . 'boo',
                    'password' => self::NW_PASS );

    $params = array( 'apikey'    => $key,
                     'ctrl_name' => '/nodeworx/siteworx',
                     'action'    => 'winCommit',
                     'input'     => $input );

    $result = $this->_apiCall( 'route', $params );
    $this->assertEquals( 'Authentication Error (NodeWorx)', $result['payload'] );
  }

  /**
   * Test.
   */
  public function testSiteworxWithApiKeyFails() {
    $input  = [];
    $params = array( 'apikey'    => $this->_getGoodApiKey(),
                     'ctrl_name' => '/siteworx/email/box',
                     'action'    => 'winCommit',
                     'input'     => $input );

    $result = $this->_apiCall( 'route', $params );
    $this->assertEquals( 'Authentication Error (SiteWorx)', $result['payload'] );
  }

  /**
   * Test.
   */
  public function testSiteworxWithLogin() {
    $input = [];
    $key   = array( 'email'    => self::SW_EMAIL,
                    'password' => self::SW_PASS,
                    'domain'   => self::SW_DOMAIN );

    $params = array( 'apikey'    => $key,
                     'ctrl_name' => '/siteworx/email/box',
                     'action'    => 'winCommit',
                     'input'     => $input );

    $result = $this->_apiCall( 'route', $params );
    $this->_assertResponseIsWinBundle( $result );
  }

  /**
   * Test.
   */
  public function testSiteworxWithBadLogin() {
    $input = [];
    $key   = array( 'email'    => self::SW_EMAIL,
                    'password' => 'WRONG',
                    'domain'   => self::SW_DOMAIN );

    $params = array( 'apikey'    => $key,
                     'ctrl_name' => '/siteworx/email/box',
                     'action'    => 'winCommit',
                     'input'     => $input );

    $result = $this->_apiCall( 'route', $params );
    $this->assertEquals( 'Authentication Error (SiteWorx)', $result['payload'] );
  }

  /**
   * Test.
   */
  public function testBasicFailure() {
    $input = [];
    $key   = array( 'email'    => self::NW_USER,
                    'password' => self::NW_PASS );

    $params = array( 'apikey'    => $key,
                     'ctrl_name' => '/nodeworx/siteworx',
                     'action'    => 'failCommit',
                     'input'     => $input );

    $result = $this->_apiCall( 'route', $params );
    $this->_assertResponseIsFailBundle( $result );
  }

  /**
   * Test.
   */
  public function testNodeworxUserEdit() {
    $nw_id = NodeWorx::getUserIdByEmail( self::NW_USER );

    $input = array( 'user' => self::NW_USER );
    $key   = array( 'email'    => self::NW_USER,
                    'password' => self::NW_PASS );

    $params = array( 'apikey'    => $key,
                     'ctrl_name' => '/nodeworx/users',
                     'action'    => 'edit',
                     'input'     => $input );

    $result = $this->_apiCall( 'route', $params );

    $this->assertEquals( API::SUCCESS, $result['status'] );
  }

  /**
   * Test.
   */
  public function testGetNextUidAndGidAction() {
    $key    = $this->_getGoodApiKey();
    $API    = new XMLRPC_API_Client( '127.0.0.1', $key );
    $result = $API->getNextUidAndGid();
    $this->assertArrayHasKey( 'uid', $result );
    $this->assertArrayHasKey( 'gid', $result );
  }
}
