<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2011 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage TESTS
 * @author     Tim College <tcollege@interworx.com>
 * @date       Nov 3, 2011
 */

/**
 * Tests changing IP addresses of a domain.
 *
 * @package    InterWorx
 * @subpackage TESTS
 */
class ChangeIpsTest extends IWorx_Framework_TestCase {
  /**
   * Setup.
   */
  static public function setUpBeforeClass() {
    self::deleteSWAccountIfNecessaryStatic();
    self::addSWAccountIfNecessary();
  }

  /**
   * Make sure the account has an IPv4.
   */
  public function testInitialSetupHasIpv4() {
    $Domain = new Domain( self::SW_DOMAIN );

    $this->assertNotEquals( false, $Domain->getIPv4Address() );
  }

  /**
   * Gets a list of IPv4s available to the addedbytest account.
   *
   * @return array
   */
  private function _getIPv4sAvailableToSW() {
    $opt = array( 'siteworx_id' => $this->getSWAccount()->getId(),
                  'version' => 4 );

    return IWorx_IP::factoryFromSearchEngine( $opt );
  }

  /**
   * Gets an IPv4 to change to.
   *
   * @param IWorx_IP $Not_This_One
   * @return IWorx_IPv4|boolean
   */
  private function _getADifferentIPv4( IWorx_IP $Not_This_One ) {
    foreach( $this->_getIPv4sAvailableToSW() as $IWorx_IP ) {
      if( $IWorx_IP->getIP() !== $Not_This_One->getIP() ) {
        return $IWorx_IP;
      }
    }

    return false;
  }

  /**
   * Gets an array of IPv6 available to the addedbytest account.
   *
   * @return array
   */
  private function _getIPv6sAvailableToSW() {
    $opt = array( 'siteworx_id' => $this->getSWAccount()->getId(),
                  'version' => 6 );

    return IWorx_IP::factoryFromSearchEngine( $opt );
  }

  /**
   * Gets a different IPv6 from the one provided.
   *
   * @param false|IWorx_IP $Not_This_One
   * @return IWorx_IP
   */
  private function _getADifferentIPv6( $Not_This_One ) {
    foreach( $this->_getIPv6sAvailableToSW() as $IWorx_IP ) {
      if( $Not_This_One === false ||
          $IWorx_IP->getIP() !== $Not_This_One->getIP() ) {
        return $IWorx_IP;
      }
    }

    return false;
  }

  /**
   * Verifies that muptiple IPv4s are available, so we can change them.
   */
  public function testInitialSetupHasMultipleIPv4Available() {
    $ips = $this->_getIPv4sAvailableToSW();
    if( count( $ips ) <= 1 ) {
      $sw_id = $this->getSWAccount()->getId();
      $DS = new DataSource_NW_IP_FreeIPsExcludingSW( IW::NW(),
                                                     $sw_id,
                                                     IWorx_IP::VERSION_4 );
      $ipv4s = $DS->getValues();
      $msg = 'Need more ipv4s available for this siteworx account';
      $this->assertGreaterThan( 1, count( $ipv4s ), $msg );
      $ip1 = $ipv4s[0];
      $ip2 = $ipv4s[1];
      $input = array( 'domain' => self::SW_DOMAIN,
                      'ipv4' => array( $ip1, $ip2 ) );
      $Reply = IW::FC()->routeFromPHP( 'Ctrl_Nodeworx_Siteworx',
                                       'addIp',
                                       $input,
                                       IW::NW() );
      $this->assertReplySuccessful( $Reply );
    }
    $ips = $this->_getIPv4sAvailableToSW();
    $this->assertGreaterThan( 1, count( $ips ), 'Only one IP available, need more' );
  }

  /**
   * Verifies that muptiple IPv6s are available, so we can change them.
   */
  public function testInitialSetupHasMultipleIPv6Available() {
    $ips = $this->_getIPv6sAvailableToSW();
    if( count( $ips ) <= 1 ) {
      $sw_id = $this->getSWAccount()->getId();
      $DS = new DataSource_NW_IP_FreeIPsExcludingSW( IW::NW(),
                                                     $sw_id,
                                                     IWorx_IP::VERSION_6 );
      $ipv6s = $DS->getValues();
      $msg = 'Need more ipv6s available for this siteworx account';
      $this->assertGreaterThan( 2, count( $ipv6s ), $msg );
      $ip1 = $ipv6s[0];
      $ip2 = $ipv6s[1];
      $input = array( 'domain' => self::SW_DOMAIN,
                      'ipv6' => array( $ip1, $ip2 ) );
      $Reply = IW::FC()->routeFromPHP( 'Ctrl_Nodeworx_Siteworx',
                                       'addIp',
                                       $input,
                                       IW::NW() );
      $this->assertReplySuccessful( $Reply );
    }
    $ips = $this->_getIPv6sAvailableToSW();
    $this->assertGreaterThan( 1, count( $ips ), 'NEED MORE IPV6, PLEASE!' );
  }

  /**
   * Make sure the account does not have an IPv6.
   */
  public function testInitialSetupDoesNotHaveIpv6() {
    $Domain = new Domain( self::SW_DOMAIN );

    $this->assertFalse( $Domain->getIPv6Address() );
  }

  /**
   * Tests that changing the IP returns true.
   */
  public function testChangingIPv4ReturnsTrue() {
    $Domain = new Domain( self::SW_DOMAIN );

    $Current_IP = $Domain->getIPv4AddressObj();
    $New_IP     = $this->_getADifferentIPv4( $Current_IP );

    $this->assertTrue( $Domain->changeIPv4AddressTo( $New_IP ) );
  }

  /**
   * Tests that changing the IP changes the Zone template replacement.
   */
  public function testChangingIPv4ChangesZoneTemplateReplacement() {
    $Domain = new Domain( self::SW_DOMAIN );

    $Current_IP = $Domain->getIPv4AddressObj();
    $New_IP     = $this->_getADifferentIPv4( $Current_IP );

    $Domain->changeIPv4AddressTo( $New_IP );

    $this->assertEquals( $New_IP->getIP(),
                         $Domain->getZone()->getTemplateIpv4Replacement() );
  }

  /**
   * Tests that changing the IP changes the DNS records.
   */
  public function testChangingIPv4UpdatesDnsRecords() {
    $Domain = new Domain( self::SW_DOMAIN );

    $Current_IP = $Domain->getIPv4AddressObj();
    $New_IP     = $this->_getADifferentIPv4( $Current_IP );

    $Domain->changeIPv4AddressTo( $New_IP );

    $q_zone_id = q( $Domain->getZone()->getId() );
    $q_ip      = q( $Current_IP->getIP() );

    $sql = "SELECT count(*) FROM dns_records
            WHERE zone_id = {$q_zone_id}
            AND target = {$q_ip}";

    $count = IWorxDbUtil::db_getOne( IW::DB(), $sql );

    $this->assertEquals( '0', $count );
  }

  /**
   * Tests that the vhost was changed.
   */
  public function testChangingIPv4UpdatesVhost() {
    $VHR = ApacheVhostReader::getInstance();
    $VHR->clearVHostCache();

    $Domain = new Domain( self::SW_DOMAIN );

    $Current_IP = $Domain->getIPv4AddressObj();
    $New_IP     = $this->_getADifferentIPv4( $Current_IP );

    $Domain->changeIPv4AddressTo( $New_IP );

    $this->assertEquals( $New_IP->getIP(), $Domain->getIPv4AddressFromSystem() );
  }

  /**
   * Tests adding an IPv6 address.
   */
  public function testAddingIPv6() {
    $Domain = new Domain( self::SW_DOMAIN );
    $New_IP = $this->_getADifferentIPv6( false );

    if( $New_IP === false ) {
      $this->markTestSkipped( 'No available IPv6 for test' );
    }

    // This has to test multiple things because we're only able to add once.
    $this->assertTrue( $Domain->setIPv6Address( $New_IP ) );
    $Domain->syncVhosts();
    ApacheVhostReader::getInstance()->clearVHostCache();

    $this->assertEquals( $New_IP->getIP(), $Domain->getIPv6AddressFromSystem() );
    $this->assertEquals( $New_IP->getIP(), $Domain->getIPv6Address() );
  }

  /**
   * Tests that changing the IP changes the Zone template replacement.
   */
  public function testChangingIPv6ChangesZoneTemplateReplacement() {
    $Domain = new Domain( self::SW_DOMAIN );

    $Current_IP = $Domain->getIPv6AddressObj();
    $New_IP     = $this->_getADifferentIPv6( $Current_IP );

    if( $New_IP === false ) {
      $this->markTestSkipped( 'No available IPv6 for test' );
    }

    $Domain->changeIPv6AddressTo( $New_IP );

    $this->assertEquals( $New_IP->getIP(),
                         $Domain->getZone()->getTemplateIpv6Replacement() );
  }

  /**
   * Tests that changing the IP returns true.
   */
  public function testChangingIPv6ReturnsTrue() {
    $Domain = new Domain( self::SW_DOMAIN );

    $Current_IP = $Domain->getIPv6AddressObj();
    $New_IP     = $this->_getADifferentIPv6( $Current_IP );

    if( $New_IP === false ) {
      $this->markTestSkipped( 'No available IPv6 for test' );
    }

    $this->assertTrue( $Domain->changeIPv6AddressTo( $New_IP ) );
  }

  /**
   * Tests that changing the IP changes the DNS records.
   */
  public function testChangingIPv6UpdatesDnsRecords() {
    $Domain = new Domain( self::SW_DOMAIN );

    $Current_IP = $Domain->getIPv6AddressObj();
    $New_IP     = $this->_getADifferentIPv6( $Current_IP );

    if( $New_IP === false ) {
      $this->markTestSkipped( 'No available IPv6 for test' );
    }

    $Domain->changeIPv6AddressTo( $New_IP );

    $q_zone_id = q( $Domain->getZone()->getId() );
    $q_ip      = q( $Current_IP->getIP() );

    $sql = "SELECT count(*) FROM dns_records
              WHERE zone_id = {$q_zone_id}
              AND target = {$q_ip}";

    $count = IWorxDbUtil::db_getOne( IW::DB(), $sql );

    $this->assertEquals( '0', $count );
  }

  /**
   * Tests that the vhost was changed.
   */
  public function testChangingIPv6UpdatesVhost() {
    $Domain = new Domain( self::SW_DOMAIN );

    $Current_IP = $Domain->getIPv6AddressObj();
    $New_IP     = $this->_getADifferentIPv6( $Current_IP );

    if( $New_IP === false ) {
      $this->markTestSkipped( 'No available IPv6 for test' );
    }

    $Domain->changeIPv6AddressTo( $New_IP );

    $this->assertEquals( $New_IP->getIP(), $Domain->getIPv6AddressFromSystem() );
  }
}
