<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2014 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Upgrade
 * @author     Tim College <tcollege@interworx.com>
 * @date       Dec 08, 2014
 */

/**
 * Class CrossAccountSubdomainTest
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class CrossAccountSubdomainTest extends IWorx_Framework_TestCase {
  const SUBDOMAIN     = 'bug3713.addedbytest.com';
  const SECOND_DOMAIN = 'bug3713number2.com';

  /**
   * Setup to clear any leftover domains.
   */
  public function setUp() {
    self::deleteSWAccountIfNecessaryStatic();
    self::deleteSWAccountIfNecessaryStatic( self::SUBDOMAIN );
    self::deleteSWAccountIfNecessaryStatic( self::SECOND_DOMAIN );
  }

  /**
   * Tests the expected behavior before the release of the 3713 bug.
   */
  public function testPre3713BehaviorAsSiteworxAccount() {
    Ini::set( Ini::SITEWORX, 'allow_crossaccount_subdomains', '1' );
    Ini::writeAndResetInstance();

    $this->addSWAccount();

    $Reply = $this->_addDomainAsSiteworxAccount( self::SUBDOMAIN );
    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Tests the expected behavior with the new configuration flag.
   */
  public function testPost3713BehaviorAsSiteworxAccount() {
    Ini::set( Ini::SITEWORX, 'allow_crossaccount_subdomains', '0' );
    Ini::writeAndResetInstance();

    $this->addSWAccount();

    $Reply = $this->_addDomainAsSiteworxAccount( self::SUBDOMAIN );
    $this->assertReplyNotSuccessful( $Reply, 'This should have failed.' );
  }

  /**
   * Ensure that regular subdomains stull function as expected.
   *
   * @depends testPost3713BehaviorAsSiteworxAccount
   */
  public function testRealSubdomainStillWorks() {
    Ini::set( Ini::SITEWORX, 'allow_crossaccount_subdomains', '0' );
    Ini::writeAndResetInstance();

    $this->addSWAccount();

    $input = ['prefix' => 'bug3713'];
    $SW    = $this->getSWAccount();

    $Reply = IW::FC()->routeFromPHP( 'Ctrl_Siteworx_DomainsSub',
                                     'addCommit',
                                     $input,
                                     $SW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Test that new configuration blocks cross-account subdomains.
   */
  public function testAddingAsSlavePostBug3713CrossAccount() {
    Ini::set( Ini::SITEWORX, 'allow_crossaccount_subdomains', '0' );
    Ini::writeAndResetInstance();

    $this->addSWAccount();
    $this->_addDomainAsSiteworxAccount( self::SECOND_DOMAIN );

    $input = ['domain' => self::SUBDOMAIN];
    $SW    = new SiteWorx( self::SECOND_DOMAIN );

    $Reply = IW::FC()->routeFromPHP( 'Ctrl_Siteworx_DomainsSlave',
                                     'addCommit',
                                     $input,
                                     $SW );

    $this->assertReplyNotSuccessful( $Reply, 'This should have failed' );
  }

  /**
   * Test that adding to the same account works as expected.
   */
  public function testAddingAsSlavePostBug3713SameAccount() {
    Ini::set( Ini::SITEWORX, 'allow_crossaccount_subdomains', '0' );
    Ini::writeAndResetInstance();

    $this->addSWAccount();

    $input = ['domain' => self::SUBDOMAIN];
    $SW    = $this->getSWAccount();

    $Reply = IW::FC()->routeFromPHP( 'Ctrl_Siteworx_DomainsSlave',
                                     'addCommit',
                                     $input,
                                     $SW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Attempt to add the subdomain as a SiteWorx account.
   *
   * @param string $domain
   * @return Reply
   */
  private function _addDomainAsSiteworxAccount( $domain ) {
    $input                  = $this->_getSWAccountInput();
    $input['master_domain'] = $domain;
    $input['uniqname']      = IWorxUtil::generate_uniqname( $domain );
    $input['nickname']      = 'Bug 3713 Unit Test';

    $NW = new NodeWorx( NodeWorx::MASTER_ID );

    $Reply = IW::FC()->routeFromPHP( 'Ctrl_Nodeworx_Siteworx',
                                     'addCommit',
                                     $input,
                                     $NW );

    return $Reply;
  }
}
