<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Paul Oehler <poehler@interworx.com>
 * @date       Feb 24, 2008
 */

/**
 * Test Suite for Ctrl_Nodeworx_Ip.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class Ctrl_Nodeworx_IpTest extends IWorx_Framework_TestCase {

  const CTRL = 'Ctrl_Nodeworx_Ip';

  const TEST_IP1 = '193.166.1.5';
  const TEST_IP2 = '193.166.1.8';

  /**
   * Data Provider.
   *
   * @return array
   */
  public function dataProviderIPs() {
    return
      array(
            array( self::TEST_IP1, self::TEST_IP2, '255.255.255.0' ),
            array( '2001:db8::d:1', '2001:db8::d:3', '64' ),
            );
  }

  /**
   * Tear down function.
   */
  public function tearDown() {
    $changed = false;

    foreach( $this->dataProviderIPs() as $range_array ) {
      $start = IPv6::toDecimal( $range_array[0] );
      $end   = IPv6::toDecimal( $range_array[1] );

      $range_version = IWorxIPUtil::getVersion( $range_array[0] );
      $is_ipv6       = ( $range_version === IWorx_IP::VERSION_6 );

      for( $ip = $start; bccomp( $ip, $end ) !== 1; $ip = bcadd( $ip, 1 ) ) {
        if( $is_ipv6 ) {
          $long_ip = IPv6::fromDecimal( $ip );
        } else {
          $long_ip = long2ip( $ip );
        }

        $IWorx_IP = IWorx_IP::factoryFromDatabase( $long_ip );

        if( $IWorx_IP !== false ) {
          $IWorx_IP->deactivate();
          $IWorx_IP->delete();
          $changed = true;
        }

        $cmd = Ini::get( Ini::IWORX_BIN, 'dns' ) . " --remove {$long_ip}";
        IWorxExec::exec( $cmd, $result, $retval, IWorxExec::KILL_STDERR );
      }//end for
    }//end foreach

    if( $changed === true ) {
      $NW = self::getMasterNodeWorx();
      $NW->getHTTPDriver()->syncIPData();
    }
  }

  /**
   * Tests if IPv6 is enabled on the box.
   */
  public function testEth0Config() {
    $Ifcfg_eth0 = new IWorx_Ini_Ifcfg( 'ifcfg-eth0' );

    $this->assertEquals( 'yes', $Ifcfg_eth0->getValue( 'IPV6INIT' ) );
  }

  /**
   * Test assigned vars of index action.
   */
  public function testIndexActionAssignedVars() {
    $this->setMaxRunningTime( 0 );
    $NW     = self::getMasterNodeWorx();
    $Reply  = IW::FC()->routeFromPHP( self::CTRL, 'index', array(), $NW );
    $result = $Reply->getData();
    $this->assertArrayHasKey( 'iw_payload', $result );
  }

  /**
   * Test.
   */
  public function testAddActionErrorsWithNoInput() {
    $NW     = self::getMasterNodeWorx();
    $Reply  = IW::FC()->routeFromPHP( self::CTRL, 'add', array(), $NW );
    $result = $Reply->getData();
    $this->assertInternalType( 'array', $result );
    $this->assertIsFormErrorArray( $result );
  }

  /**
   * Tests that adding a single IP works properly.
   *
   * @param string $start_ip
   * @param string $end_ip
   * @param string $netmask
   * @dataProvider dataProviderIPs
   */
  public function testAddAction( $start_ip, $end_ip, $netmask ) {
    $NW = self::getMasterNodeWorx();

    $input = array( 'ip_start' => $start_ip,
                    'ip_end'   => $start_ip,
                    'netmask'  => $netmask,
                    'device'   => 'eth0',
                    'type'     => IWorx_IP::STATICIP,
                    'reseller' => 0,
                    'vip'      => 0 );

    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'add', $input, $NW );

    $this->assertReplySuccessful( $Reply );
    $this->assertTrue( IWorxIPUtil::existsInDb( $start_ip ) );

    $tdnsbasedir = Ini::get( Ini::TINYDNS_DIR, 'base' );
    $service     = Ini::get( Ini::TINYDNS_DIR, 'supervise' );
    $ip          = $start_ip;

    $target1 = "$service/axfrdns-$ip";
    $target2 = "$tdnsbasedir/axfrdns-$ip";

    $this->assertTrue( IWorxFileSys::fileExists( $target1 ) );
    $this->assertTrue( IWorxFileSys::fileExists( $target2 ) );
  }

  /**
   * Tests that an SSL domain IP can switch types.
   */
  public function testEditSslDomainIpTypeAction() {
    // Add SW account
    $this->deleteSWAccountIfNecessary();
    $this->addSWAccount( true );

    // Get our IP
    $NW = self::getMasterNodeWorx();
    $SW = self::getSWAccount();
    $ip = self::TEST_IP1;

    $input = array( 'ip_start' => $ip,
                    'ip_end'   => $ip,
                    'netmask'  => '255.255.255.0',
                    'device'   => 'eth0',
                    'type'     => IWorx_IP::STATICIP,
                    'reseller' => 0,
                    'vip'      => 0 );

    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'add', $input, $NW );
    $this->assertReplySuccessful( $Reply );

    $input = array( 'ip' => $ip,
                    'is_static' => true,
                    'is_reseller' => true );
    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'edit', $input, $NW );
    $this->assertReplySuccessful( $Reply );
    $this->assertTrue( IWorxIPUtil::existsInDb( $ip ) );

    $IWorx_IP = IWorx_IP::factoryFromDatabase( $ip );
    $this->assertTrue( $IWorx_IP->isStatic() );

    // Add SSL to SW account
    $Reply = IW::FC()->routeFromPHP( 'Ctrl_Siteworx_SslKey',
                                     'generate',
                                     array( 'key_length' => '2048' ),
                                     $SW );
    $this->assertReplySuccessful( $Reply );

    $input = array( 'countryName'            => 'US',
                    'stateOrProvinceName'    => 'Pennsylvania',
                    'localityName'           => 'Pittsburgh',
                    'organizationName'       => 'InterWorx, LLC',
                    'organizationalUnitName' => 'SE',
                    'commonName'             => self::SW_DOMAIN,
                    'emailAddress'           => self::SW_EMAIL );
    $Reply = IW::FC()->routeFromPHP( 'Ctrl_Siteworx_SslCsr',
                                     'generate',
                                     $input,
                                     $SW );
    $this->assertReplySuccessful( $Reply );

    $Reply = IW::FC()->routeFromPHP( 'Ctrl_Siteworx_SslCrt',
                                     'generate',
                                     array(),
                                     $SW );
    $this->assertReplySuccessful( $Reply );
    ApacheVhostReader::getInstance()->clearVHostCache();
    $this->assertTrue( $SW->getMasterDomain()->isSSL() );

    // Switch SW account IP from dedicated to shared
    $input = array( 'ip' => $ip,
                    'is_static' => 0,
                    'is_reseller' => true );
    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'edit', $input, $NW );
    $this->assertReplySuccessful( $Reply );

    // Switch SW account IP from shared to dedicated
    $input = array( 'ip' => $ip,
                    'is_static' => 1,
                    'is_reseller' => true );
    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'edit', $input, $NW );
    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Tests that adding a range works properly.
   *
   * @param string $start_ip
   * @param string $end_ip
   * @param string $netmask
   * @dataProvider dataProviderIPs
   */
  public function testAddActionForIpRange( $start_ip, $end_ip, $netmask ) {
    $NW = self::getMasterNodeWorx();

    $input = array( 'ip_start' => $start_ip,
                    'ip_end'   => $end_ip,
                    'netmask'  => $netmask,
                    'device'   => 'eth0',
                    'type'     => IWorx_IP::STATICIP,
                    'reseller' => 0,
                    'vip'      => 0 );

    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'add', $input, $NW );

    $this->assertReplySuccessful( $Reply );
    $this->assertTrue( IWorxIPUtil::existsInDb( $start_ip ) );
    $this->assertTrue( IWorxIPUtil::existsInDb( $end_ip ) );
  }

  /**
   * Test.
   */
  public function testResellersActionErrorsWithNoInput() {
    $NW    = self::getMasterNodeWorx();
    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'listResellers', array(), $NW );
    $this->assertFalse( $Reply->wasSuccessful() );
  }

  /**
   * Test.
   */
  public function testDomainsActionErrorsWithNoInput() {
    $NW     = self::getMasterNodeWorx();
    $Reply  = IW::FC()->routeFromPHP( self::CTRL, 'listDomains', array(), $NW );
    $this->assertFalse( $Reply->wasSuccessful() );
  }
}
