<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Tim College <tcollege@interworx.com>
 * @date       Feb 26, 2009
 */

/**
 * Tests for Ctrl_Siteworx_DomainsPointer.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class Ctrl_Siteworx_DomainsPointerTest extends IWorx_Framework_TestCase {
  const CTRL = 'Ctrl_Siteworx_DomainsPointer';

  /**
   * Creates a fresh account if needed.
   */
  public function testSetup() {
    $data = $this->getSWIndexData( self::CTRL );

    if( count( $data['iw_payload']->getData() ) !== 0 ) {
      $this->deleteSWAccountIfNecessary();
    }
  }

  /**
   * Tests the output of the index action.
   */
  public function testIndexActionReturnsArray() {
    $output = $this->getSWIndexData( self::CTRL );

    $this->assertInternalType( 'array', $output );
  }

  /**
   * Tests the initial count of pointers.
   */
  public function testInitialPointerCountIsZero() {
    $data = $this->getSWIndexData( self::CTRL );

    $this->assertCount( 0, $data['iw_payload']->getData(),
                        'Pointer domains already exist!' );
  }

  /**
   * Tests that addPointerAction fails without input.
   */
  public function testAddFailsWithoutInput() {
    $SW = $this->getSWAccount();

    $Reply  = IW::FC()->routeFromPHP( self::CTRL, 'add', array(), $SW );
    $result = $Reply->getData();

    $this->assertIsFormErrorArray( $result );
  }

  /**
   * Tests that addAction returns true.
   */
  public function testAddReturnsTrue() {
    $SW = $this->getSWAccount();

    $input = array( 'domain' => self::SW_POINTER_DOMAIN,
                    'points_to' => self::SW_DOMAIN );

    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'add', $input, $SW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Tests the new count of pointers.
   */
  public function testNewPointerCountIsOne() {
    $data = $this->getSWIndexData( self::CTRL );

    $this->assertCount( 1, $data['iw_payload']->getData(),
                        'Pointer domain does not seem to have been added' );
  }

  /**
   * Tests that deleteAction fails without input.
   */
  public function testDeleteFailsWithoutInput() {
    $SW = $this->getSWAccount();

    $Reply  = IW::FC()->routeFromPHP( self::CTRL, 'delete', array(), $SW );
    $result = $Reply->getData();

    $this->assertIsFormErrorArray( $result );
  }

  /**
   * Tests that deleteAction returns true.
   */
  public function testDeleteReturnsTrue() {
    $SW = $this->getSWAccount();

    $input = array( 'domain' => self::SW_POINTER_DOMAIN );

    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'delete', $input, $SW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Tests the deleted count of pointers.
   */
  public function testPointerCountIsZero() {
    $data = $this->getSWIndexData( self::CTRL );

    $this->assertCount( 0, $data['iw_payload']->getData(),
                        'Pointer domain does not seem to have been deleted' );
  }

  /**
   * Tests that the SOA record was properly generated.
   */
  public function testSOARecordInherited() {
    $NW = $this->getMasterNodeWorx();

    // delete SW
    $this->deleteSWAccountIfNecessary();

    // modify NW SOA
    $NWSOA = $NW->getDNSTemplateZoneObj()->getSoaRecord();
    $NWSOA->setEmail( 'testmaster@{domain}' );
    $NWSOA->save();

    // create SW
    $this->addSWAccount();

    // verify SW SOA
    $SW     = $this->getSWAccount();
    $where  = 'type="' . DNS_Record::SOA . '"';
    $SOA    = $SW->getDomainZoneRecords( $where );
    $target = $SOA[0][0]['target'];
    $this->assertContains( 'testmaster', $target );

    // create pointer
    $input = array( 'domain' => self::SW_POINTER_DOMAIN,
                    'points_to' => self::SW_DOMAIN );
    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'add', $input, $SW );
    $this->assertReplySuccessful( $Reply );

    // verify pointer SOA
    $where  .= ' AND host="' . self::SW_POINTER_DOMAIN . '"';
    $SOA    = $SW->getDomainZoneRecords( $where );
    $target = $SOA[0][0]['target'];
    $this->assertContains( 'testmaster', $target );

    $input = array( 'domain' => self::SW_POINTER_DOMAIN );
    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'delete', $input, $SW );
    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Tests that vhosts are updated properly on deletion.
   */
  public function testPointerVhostUpdates() {
    $SW     = $this->getSWAccount();
    $domain = self::SW_DOMAIN;
    $ptrdom = self::SW_POINTER_DOMAIN;
    $cat    = Ini::get( Ini::BIN, 'cat' );
    $grep   = Ini::get( Ini::BIN, 'grep' );

    $input = array( 'domain' => $ptrdom,
                    'points_to' => self::SW_DOMAIN );
    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'add', $input, $SW );
    $this->assertReplySuccessful( $Reply );

    $vhost = Ini::get( Ini::APACHE_DIR, 'conf.d' ) . "/vhost_{$domain}.conf";
    $cmd   = "{$cat} {$vhost} | {$grep} {$ptrdom}";

    $Command = new Command( $cmd );
    $Command->allowPipe();
    $Command->allowMultipleExecution();
    $Command->exec();
    $this->assertTrue( $Command->wasSuccessful() );

    $input = array( 'domain' => $ptrdom );
    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'delete', $input, $SW );
    $this->assertReplySuccessful( $Reply );

    $Command->exec();
    $this->assertFalse( $Command->wasSuccessful() );
  }
}
