<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Tim College <tcollege@interworx.com>
 * @date       Feb 26, 2009
 */

/**
 * Tests for Ctrl_Siteworx_Ftp.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class Ctrl_Siteworx_FtpTest extends IWorx_Framework_TestCase {
  const CTRL = 'Ctrl_Siteworx_Ftp';

  const USER = 'zftpuser';
  const PASS = 'password_password';

  /**
   * Deletes the account if necessary.
   */
  public function testSetup() {
    $data = $this->getSWIndexData( self::CTRL );

    if( count( $data['iw_payload']->getData() ) !== 1 ) {
      $this->deleteSWAccountIfNecessary();
    }
  }

  /**
   * Tests the output of the index action.
   */
  public function testIndexActionReturnsArray() {
    $output = $this->getSWIndexData( self::CTRL );

    $this->assertInternalType( 'array', $output );
  }

  /**
   * Verifies that only the master ftp account exists initially.
   */
  public function testInitialFtpAccountCount() {
    $data = $this->getSWIndexData( self::CTRL );

    $this->assertCount( 1, $data['iw_payload']->getData(),
                        'Unexpected count of FTP accounts' );
  }

  /**
   * Tests that addAction fails without input.
   */
  public function testAddFailsWithoutInput() {
    $SW = $this->getSWAccount();

    $Reply  = IW::FC()->routeFromPHP( self::CTRL, 'add', array(), $SW );
    $result = $Reply->getData();

    $this->assertIsFormErrorArray( $result );
  }

  /**
   * Tests that addAction returns true.
   */
  public function testAddReturnsTrue() {
    $SW = $this->getSWAccount();

    $input = array( 'user'             => self::USER,
                    'password'         => self::PASS,
                    'confirm_password' => self::PASS,
                    'homedir'          => '/home/' . self::SW_UNIQNAME );

    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'add', $input, $SW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Test that the add action seemed to work.
   */
  public function testAddActionWorked() {
    $data = $this->getSWIndexData( self::CTRL );

    $this->assertCount( 2, $data['iw_payload']->getData() );
  }

  /**
   * Tests that editAction fails without input.
   */
  public function testEditFailsWithoutInput() {
    $SW = $this->getSWAccount();

    $Reply  = IW::FC()->routeFromPHP( self::CTRL, 'edit', array(), $SW );
    $result = $Reply->getData();

    $this->assertIsFormErrorArray( $result );
  }

  /**
   * Tests that editAction returns true.
   */
  public function testEditReturnsTrue() {
    $SW = $this->getSWAccount();

    $input = array( 'user' => self::USER,
                    'homedir' => '/home/' .
                    self::SW_UNIQNAME .
                    '/' .
                    $SW->getWorkingDomainName() );

    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'edit', $input, $SW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Test that the edit action seemed to work.
   */
  public function testEditActionWorked() {
    $SW = $this->getSWAccount();

    $expected = '/home/' . self::SW_UNIQNAME . '/' . $SW->getWorkingDomainName();

    $data = $this->getSWIndexData( self::CTRL );

    foreach( $data['iw_payload']->getData() as $account ) {
      if( $account->username === self::USER ) {
        $user = $account;
      }
    }

    $this->assertEquals( $expected, $user->homedir );
  }

  /**
   * Tests that suspendAction fails without input.
   */
  public function testSuspendFailsWithoutInput() {
    $SW = $this->getSWAccount();

    $Reply  = IW::FC()->routeFromPHP( self::CTRL, 'suspend', array(), $SW );
    $result = $Reply->getData();

    $this->assertIsFormErrorArray( $result );
  }

  /**
   * Tests that unsuspendAction fails without input.
   */
  public function testUnsuspendFailsWithoutInput() {
    $SW = $this->getSWAccount();

    $Reply  = IW::FC()->routeFromPHP( self::CTRL, 'unsuspend', array(), $SW );
    $result = $Reply->getData();

    $this->assertIsFormErrorArray( $result );
  }

  /**
   * Tests that suspendAction returns true.
   */
  public function testSuspendReturnsTrue() {
    $SW = $this->getSWAccount();

    $input = array( 'user' => self::USER );

    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'suspend', $input, $SW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Test that the suspend action seemed to work.
   */
  public function testSuspendActionWorked() {
    $Account = $this->getSWAccount()->getFTPDriver()->getAccount( self::USER );
    $this->assertEquals( $Account->status, ProFTP::INACTIVE );
  }

  /**
   * Tests that unsuspendAction returns true.
   *
   * @depends testSuspendActionWorked
   */
  public function testUnsuspendReturnsTrue() {
    $SW = $this->getSWAccount();

    $input = array( 'user' => self::USER );

    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'unsuspend', $input, $SW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Test that the unsuspend action seemed to work.
   *
   * @depends testSuspendActionWorked
   */
  public function testUnsuspendActionWorked() {
    $Account = $this->getSWAccount()->getFTPDriver()->getAccount( self::USER );
    $this->assertEquals( $Account->status, ProFTP::ACTIVE );
  }

  /**
   * Tests that deleteAction fails without input.
   */
  public function testDeleteFailsWithoutInput() {
    $SW = $this->getSWAccount();

    $Reply  = IW::FC()->routeFromPHP( self::CTRL, 'delete', array(), $SW );
    $result = $Reply->getData();

    $this->assertIsFormErrorArray( $result );
  }

  /**
   * Tests that deleteAction returns true.
   */
  public function testDeleteReturnsTrue() {
    $SW = $this->getSWAccount();

    $input = array( 'user' => self::USER );

    $Reply = IW::FC()->routeFromPHP( self::CTRL, 'delete', $input, $SW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Test that the delete action seemed to work.
   */
  public function testDeleteActionWorked() {
    $data = $this->getSWIndexData( self::CTRL );

    $this->assertCount( 1, $data['iw_payload']->getData() );
  }
}
