<?php

/**
 * InterWorx Hosting Control Panel
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2017 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Upgrade
 * @author     Tim College <tcollege@interworx.com>
 * @date       Apr 17, 2017
 */

/**
 * Class DomainPromoterPhpModeTest
 *
 * @package    InterWorx
 * @subpackage TESTS
 */
class DomainPromoterPhpModeTest extends IWorx_Framework_TestCase {
  /**
   * Prepare.
   */
  public function setUp() {
    self::deleteSWAccountIfNecessaryStatic();
    self::deleteSWAccountIfNecessaryStatic( self::SW_SECONDARY_DOMAIN );
    self::deleteSWAccountIfNecessaryStatic( self::SW_POINTER_DOMAIN );
    IWorxLog::log( 'Sites apparently cleaned up, moving on' );
    $this->_waitForCommandQueueToCatchUp();
  }

  /**
   * Test the initial assumptions.
   */
  public function testInitialAssumptions() {
    $this->_setPhpMode( PHP::MODE_MOD_SUPHP );
    PHP::setDefaultVersion( PHP::SYSTEM_PHP );

    $SW = $this->getSWAccount();

    $this->assertEquals( PHP::SYSTEM_PHP, $SW->getPhpVersion() );
  }

  /**
   * Test things work when the default PHP is the same as the SiteWorx php.
   */
  public function testPromoteWithSameSiteworxPhpAsDefaultSuphp() {
    $this->_setPhpMode( PHP::MODE_MOD_SUPHP );
    PHP::setDefaultVersion( PHP::SYSTEM_PHP );

    $SW = $this->getSWAccount();
    $this->_addPointerDomain( $SW );

    $this->_promote( 'pointer', self::SW_DOMAIN, self::SW_POINTER_DOMAIN );

    $PromotedSW = new SiteWorx( self::SW_POINTER_DOMAIN );

    $this->assertEquals( PHP::SYSTEM_PHP, $PromotedSW->getPhpVersion() );

    $pointer    = self::SW_POINTER_DOMAIN;
    $vhost_file = Ini::get( Ini::APACHE_DIR, 'conf.d' ) . "/vhost_{$pointer}.conf";

    $this->assertFileContains( $vhost_file, 'iw-php-mode: system-mod-suphp' );
  }

  /**
   * Test with a different SiteWorx PHP.
   */
  public function testPromoteWithDifferentSiteworxPhpAsDefaultSuphp() {
    $this->_setPhpMode( PHP::MODE_MOD_SUPHP );
    PHP::setDefaultVersion( PHP::SYSTEM_PHP );

    if( PHP::isMultiplePhpInstalled() === false ) {
      PHP::enableMultiplePhp();
    }

    $sw_php = '/opt/remi/php71';

    $Php = new PhpFpm( $sw_php );
    if( $Php->isInstalled() === false ) {
      PHP::install( $sw_php );
    }

    if( $Php->isEnabled() === false ) {
      PHP::setEnabledVersions( array_merge( PHP::getEnabledVersions(),
                                            [$sw_php] ) );
    }

    $input = $this->_getSWAccountInput();
    $input[ Form_NW_Siteworx_Add::INPUT_PHP_VERSION ] = $sw_php;

    $this->addSWAccountFromInput( $input );

    $SW = $this->getSWAccount();
    $this->_addPointerDomain( $SW );

    $this->assertEquals( $sw_php, $SW->getPhpVersion() );

    $this->_promote( 'pointer', self::SW_DOMAIN, self::SW_POINTER_DOMAIN );

    $PromotedSW = new SiteWorx( self::SW_POINTER_DOMAIN );

    $this->assertEquals( $sw_php, $PromotedSW->getPhpVersion() );

    $pointer    = self::SW_POINTER_DOMAIN;
    $vhost_file = Ini::get( Ini::APACHE_DIR, 'conf.d' ) . "/vhost_{$pointer}.conf";

    $this->assertFileContains( $vhost_file, 'iw-php-mode: php-fpm' );
    $this->assertFileContains( $vhost_file, $sw_php );
  }

  /**
   * Kinda dumb test, but helpful for my debugging.
   */
  public function testCommandQueueStatus() {
    if( IW::Env()->isManager() ) {
      $this->assertFalse( IW::CommandQueue()->hasStoppedNodes(),
                          'The Command Queue is offline! Boo!' );
    }
  }

  /**
   * Set the php mode.
   *
   * @param string $mode
   */
  private function _setPhpMode( $mode ) {
    $this->assertReplySuccessful( self::_setPhpModeStatic( $mode ) );
  }

  /**
   * Static method to set PHP mode.
   *
   * @param string $mode
   * @return Reply
   */
  static private function _setPhpModeStatic( $mode ) {
    return IW::FC()->routeFromPHP( 'Ctrl_Nodeworx_Http',
                                   'updatePhpModeCommit',
                                   ['php_mode' => $mode],
                                   self::getMasterNodeWorx() );
  }

  /**
   * Add a pointer domain to the SiteWorx account.
   *
   * @param SiteWorx $SW
   * @param string   $pointer
   * @param string   $type
   */
  private function _addPointerDomain( SiteWorx $SW,
                                      $pointer = self::SW_POINTER_DOMAIN,
                                      $type = Domain::PTR_ALIAS ) {
    $input = ['domain' => $pointer, 'redir_type' => $type];

    $Reply = IW::FC()
      ->routeFromPHP( 'Ctrl_Siteworx_DomainsPointer', 'add', $input, $SW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Promote a domain.
   *
   * @param string $promo_type
   * @param string $primary
   * @param string $other_domain
   * @throws IWorx_Exception If unknown promo type.
   */
  private function _promote( $promo_type, $primary, $other_domain ) {
    $this->_waitForCommandQueueToCatchUp();

    $cmd = ['/home/interworx/bin/domain-promoter.pex'];

    if( $promo_type === 'pointer' ) {
      $cmd[]     = '--pointer-to-primary';
      $other_arg = '--pointer-domain';
    } else if( $promo_type === 'secondary' ) {
      $cmd[]     = '--secondary-to-primary';
      $other_arg = '--secondary-domain';
    } else {
      throw new IWorx_Exception( 'Unknown promotion type',
                                 Reply::ERROR_YOURE_DOING_IT_WRONG );
    }

    $cmd[] = "--primary-domain {$primary}";

    $cmd[] = $other_arg;
    $cmd[] = $other_domain;

    $cmd[] = '--non-interactive';

    $Cmd = new Command( implode( ' ', $cmd ) );
    $Cmd->exec();

    if( $Cmd->wasSuccessful() === false ) {
      $Cmd->logResult();
      echo $Cmd->getResultMessage();
    }

    $this->assertTrue( $Cmd->wasSuccessful(), 'domain promoter failed' );
  }
}
