<?php
/**
 * InterWorx Hosting Control Panel
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2017 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Upgrade
 * @author     Tim College <tcollege@interworx.com>
 * @date       Apr 06, 2017
 */

/**
 * Class DomainPromoterTest
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class DomainPromoterTest extends IWorx_Framework_TestCase {
  const CONFIG_NAME_SW_POINTER       = 'pointer-promote-test';
  const CONFIG_NAME_DOMAIN_POINTER   = 'pointer-domain-config-value';
  const CONFIG_NAME_SW_SECONDARY     = 'secondary-promote-test';
  const CONFIG_NAME_DOMAIN_SECONDARY = 'config-test-secondary-domain';

  const CONFIG_VALUE_SW      = 'siteworx-level-value';
  const CONFIG_VALUE_DOMAIN  = 'domain-level-value-1';
  const CONFIG_VALUE_DOMAIN2 = 'domain-level-value-2';

  /**
   * Test promoting a pointer domain.
   */
  public function testPointerToPrimaryPromotion() {
    $this->deleteSWAccountIfNecessary();
    $SW = $this->getSWAccount();
    $this->_addPointerDomain( $SW );
    $SW->setConfig( self::CONFIG_NAME_SW_POINTER . '', self::CONFIG_VALUE_SW );
    $Domain = $SW->getMasterDomain();
    $Domain->setConfig( self::CONFIG_NAME_DOMAIN_POINTER,
                        self::CONFIG_VALUE_DOMAIN );

    $this->_promote( 'pointer',
                     $SW->getMasterDomainName(),
                     self::SW_POINTER_DOMAIN );
  }

  /**
   * Test that the domain types are correct.
   *
   * @depends testPointerToPrimaryPromotion
   */
  public function testPointerToPrimaryDomainTypes() {
    $NewPrimary = new Domain( self::SW_POINTER_DOMAIN );
    $this->assertEquals( Domain::MASTER, $NewPrimary->getType() );

    $OldPrimary = new Domain( self::SW_DOMAIN );
    $this->assertEquals( Domain::POINTER, $OldPrimary->getType() );
  }

  /**
   * Test that the vhost files look correct.
   *
   * @depends testPointerToPrimaryPromotion
   */
  public function testPointerToPrimaryVhostFiles() {
    $old = self::SW_DOMAIN;
    $new = self::SW_POINTER_DOMAIN;

    $old_file = Ini::get( Ini::APACHE_DIR, 'conf.d' ) . "/vhost_{$old}.conf";
    $new_file = Ini::get( Ini::APACHE_DIR, 'conf.d' ) . "/vhost_{$new}.conf";

    $this->assertFileExists( $new_file );
    $this->assertFileNotExists( $old_file );
  }

  /**
   * Test that the directories have been renamed.
   *
   * @depends testPointerToPrimaryPromotion
   */
  public function testPointerToPrimaryDirectoryNames() {
    $old  = self::SW_DOMAIN;
    $new  = self::SW_POINTER_DOMAIN;
    $user = self::SW_UNIQNAME;
    $home = Ini::get( Ini::IWORX_DIR, 'client_home' );

    $this->assertFileExists( "{$home}/{$user}/{$new}" );
    $this->assertFileNotExists( "{$home}/{$user}/{$old}}" );

    $this->assertFileExists( "{$home}/{$user}/var/{$new}" );
    $this->assertFileNotExists( "{$home}/{$user}/var/{$old}}" );
  }

  /**
   * Test that config data was moved correctly for the SiteWorx account.
   */
  public function testPointerToPrimarySiteWorxConfig() {
    $SW = $this->getSWAccount();
    $this->assertEquals( self::SW_POINTER_DOMAIN, $SW->getConfigKey() );
    $this->assertEquals( self::CONFIG_VALUE_SW,
                         $SW->getConfig( self::CONFIG_NAME_SW_POINTER ) );
  }

  /**
   * Test the config data for the domain.
   */
  public function testPointerToPrimaryDomainConfig() {
    $Domain = $this->getSWAccount()->getMasterDomain();

    // For pointers, we want config values assigned to the master domain to become
    // associated with the new master domain.  Pointers don't really have their own
    // config, though it's technically possible.
    $this->assertEquals( self::CONFIG_VALUE_DOMAIN,
                         $Domain->getConfig( self::CONFIG_NAME_DOMAIN_POINTER ) );
  }

  /**
   * Test promoting a secondary domain.
   */
  public function testSecondaryToPrimaryPromotion() {
    $this->deleteSWAccountIfNecessary();
    $SW = $this->getSWAccount();
    $this->_addSecondaryDomain( $SW );
    $SW->setConfig( self::CONFIG_NAME_SW_SECONDARY, self::CONFIG_VALUE_SW );

    $Primary   = $SW->getMasterDomain();
    $Secondary = $SW->getDomain( self::SW_SECONDARY_DOMAIN );

    $Primary->setConfig( self::CONFIG_NAME_DOMAIN_SECONDARY,
                         self::CONFIG_VALUE_DOMAIN );
    $Secondary->setConfig( self::CONFIG_NAME_DOMAIN_SECONDARY,
                           self::CONFIG_VALUE_DOMAIN2 );

    $this->_promote( 'secondary',
                     $SW->getMasterDomainName(),
                     self::SW_SECONDARY_DOMAIN );
  }

  /**
   * Test that the domain types are correct.
   *
   * @depends testSecondaryToPrimaryPromotion
   */
  public function testSecondaryToPrimaryDomainTypes() {
    $NewPrimary = new Domain( self::SW_SECONDARY_DOMAIN );
    $this->assertEquals( Domain::MASTER, $NewPrimary->getType() );

    $OldPrimary = new Domain( self::SW_DOMAIN );
    $this->assertEquals( Domain::SLAVE, $OldPrimary->getType() );
  }

  /**
   * Check up on the parent ids, make sure that was reassigned correctly.
   */
  public function testSecondaryToPrimaryParentDomainIds() {
    $NewPrimary = new Domain( self::SW_SECONDARY_DOMAIN );
    $OldPrimary = new Domain( self::SW_DOMAIN );

    $this->assertFalse( $NewPrimary->getParentDomain() );
    $this->assertEquals( $NewPrimary->getId(),
                         $OldPrimary->getParentDomain()->getId() );
  }

  /**
   * Test that the vhost files look correct.
   *
   * @depends testSecondaryToPrimaryPromotion
   */
  public function testSecondaryToPrimaryVhostFiles() {
    $old = self::SW_DOMAIN;
    $new = self::SW_SECONDARY_DOMAIN;

    $old_file = Ini::get( Ini::APACHE_DIR, 'conf.d' ) . "/vhost_{$old}.conf";
    $new_file = Ini::get( Ini::APACHE_DIR, 'conf.d' ) . "/vhost_{$new}.conf";

    $this->assertFileExists( $new_file );
    $this->assertFileExists( $old_file );
  }

  /**
   * Test that config data was moved correctly for the SiteWorx account.
   */
  public function testSecondaryToPrimarySiteWorxConfig() {
    $SW = $this->getSWAccount();
    $this->assertEquals( self::SW_SECONDARY_DOMAIN, $SW->getConfigKey() );
    $this->assertEquals( self::CONFIG_VALUE_SW,
                         $SW->getConfig( self::CONFIG_NAME_SW_SECONDARY ) );
  }

  /**
   * Test domain config after promotion.
   */
  public function testSecondaryToPrimaryDomainConfig() {
    $SW        = $this->getSWAccount();
    $Primary   = $SW->getMasterDomain();
    $Secondary = $SW->getDomain( self::SW_DOMAIN );

    // For secondary domains, we want the config values to stay with the domain, not
    // to be moved around.
    $this->assertEquals( self::CONFIG_VALUE_DOMAIN2,
                         $Primary->getConfig( self::CONFIG_NAME_DOMAIN_SECONDARY ) );

    $config = $Secondary->getConfig( self::CONFIG_NAME_DOMAIN_SECONDARY );
    $this->assertEquals( self::CONFIG_VALUE_DOMAIN, $config );
  }

  /**
   * Kinda dumb test, but helpful for my debugging.
   */
  public function testCommandQueueStatus() {
    if( IW::Env()->isManager() ) {
      $this->assertFalse( IW::CommandQueue()->hasStoppedNodes(),
                          'The Command Queue is offline! Boo!' );
    }
  }

  /**
   * Add a secondary domain to the SiteWorx account.
   *
   * @param SiteWorx $SW
   * @param string   $secondary
   */
  private function _addSecondaryDomain( SiteWorx $SW,
                                        $secondary = self::SW_SECONDARY_DOMAIN ) {
    $input = ['domain' => $secondary];

    $Reply = IW::FC()
      ->routeFromPHP( 'Ctrl_Siteworx_DomainsSlave', 'add', $input, $SW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Add a pointer domain to the SiteWorx account.
   *
   * @param SiteWorx $SW
   * @param string   $pointer
   * @param string   $type
   */
  private function _addPointerDomain( SiteWorx $SW,
                                      $pointer = self::SW_POINTER_DOMAIN,
                                      $type = Domain::PTR_ALIAS ) {
    $input = ['domain' => $pointer, 'redir_type' => $type];

    $Reply = IW::FC()
      ->routeFromPHP( 'Ctrl_Siteworx_DomainsPointer', 'add', $input, $SW );

    $this->assertReplySuccessful( $Reply );
  }

  /**
   * Promote a domain.
   *
   * @param string $promo_type
   * @param string $primary
   * @param string $other_domain
   * @throws IWorx_Exception If unknown promo type.
   */
  private function _promote( $promo_type, $primary, $other_domain ) {
    $this->_waitForCommandQueueToCatchUp();
    $cmd = ['/home/interworx/bin/domain-promoter.pex'];

    if( $promo_type === 'pointer' ) {
      $cmd[]     = '--pointer-to-primary';
      $other_arg = '--pointer-domain';
    } else if( $promo_type === 'secondary' ) {
      $cmd[]     = '--secondary-to-primary';
      $other_arg = '--secondary-domain';
    } else {
      throw new IWorx_Exception( 'Unknown promotion type',
                                 Reply::ERROR_YOURE_DOING_IT_WRONG );
    }

    $cmd[] = "--primary-domain {$primary}";

    $cmd[] = $other_arg;
    $cmd[] = $other_domain;

    $cmd[] = '--non-interactive';

    $Cmd = new Command( implode( ' ', $cmd ) );
    $Cmd->exec();

    if( $Cmd->wasSuccessful() === false ) {
      $Cmd->logResult();
      echo $Cmd->getResultMessage();
    }

    $this->assertTrue( $Cmd->wasSuccessful(), 'domain promoter failed' );
  }
}
