<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2013 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Tim College <tcollege@interworx.com>
 * @date       May 19, 2013
 */

/**
 * Tests for maintenance mode.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class MaintenanceModeTest extends IWorx_Framework_TestCase {
  /**
   * Clears out any existing maintenance mode flags.
   */
  public function setup() {
    $cmd = 'rm -f /home/interworx/etc/env/maintenance-*';
    IWorxExec::exec( $cmd, $result, $retval, IWorxExec::ALLOW_ASTERISK );
  }

  /**
   * Clears out any existing maintenance mode flags.
   */
  public function tearDown() {
    $cmd = 'rm -f /home/interworx/etc/env/maintenance-*';
    IWorxExec::exec( $cmd, $result, $retval, IWorxExec::ALLOW_ASTERISK );
  }

  /**
   * Tests that maintenance mode is initially off.
   */
  public function testMaintenanceModeIsOff() {
    $this->assertFalse( IW::Env()->isInMaintenanceMode() );
  }

  /**
   * Tests turning on maintenance mode.
   */
  public function testMaintenanceModeTurnOn() {
    IW::Env()->setMaintenanceMode( 'unit-tests' );
    $this->assertTrue( IW::Env()->isInMaintenanceMode() );
  }

  /**
   * Checks that turning maintenance mode off works.
   */
  public function testClearMaintenanceMode() {
    IW::Env()->setMaintenanceMode( 'unit-tests' );
    IW::Env()->clearMaintenanceMode( 'unit-tests' );
    $this->assertFalse( IW::Env()->isInMaintenanceMode() );
  }

  /**
   * Tests that CLI returns the right value when maintenance mode is on.
   */
  public function testCLIReturnValue() {
    IW::Env()->setMaintenanceMode( 'unit-tests' );

    $result = [];
    $retval = -1;
    $cmd    = Ini::get( Ini::IWORX_BIN, 'command-queue' );
    $cmd    = "{$cmd} --status";

    IWorxExec::exec( $cmd, $result, $retval );

    $this->assertEquals( CLI::MAINTENANCE, $retval );
  }

  /**
   * Checks that the special flag works in CLI.
   */
  public function testCLIReturnValueWithSpecialFlag() {
    IW::Env()->setMaintenanceMode( 'unit-tests' );

    $result = [];
    $retval = -1;
    $cmd    = Ini::get( Ini::IWORX_BIN, 'command-queue' );
    $cmd    = "{$cmd} --status --run-in-maintenance-mode";

    IWorxExec::exec( $cmd, $result, $retval );

    $this->assertNotEquals( CLI::MAINTENANCE, $retval );
  }

  /**
   * Tests that the license cron keeps working.
   */
  public function testCLILicenseCronWithoutFlag() {
    IW::Env()->setMaintenanceMode( 'unit-tests' );

    $result = [];
    $retval = -1;
    $cmd    = Ini::get( Ini::IWORX_CRON, 'license' );
    $cmd    = "{$cmd} --sync";

    IWorxExec::exec( $cmd, $result, $retval );

    $this->assertNotEquals( -1, $retval );
    $this->assertNotEquals( CLI::MAINTENANCE, $retval );
  }

  /**
   * Checks that the API will return the maintenance code.
   */
  public function testAPIReturnValue() {
    IW::Env()->setMaintenanceMode( 'unit-tests' );

    $url = 'http://127.0.0.1:2080/nodeworx/xmlrpc-api.php';

    $Client = new Zend_XmlRpc_Client( $url );
    $key    = array( 'email' => self::NW_USER, 'password' => self::NW_PASS );

    $params = array( 'apikey'    => $key,
                     'ctrl_name' => 'nodeworx/index',
                     'action'    => 'winCommit',
                     'input'     => array() );

    $result = $Client->call( 'iworx.route', $params );

    $this->assertEquals( API::ERROR_SERVER_MAINTENANCE, $result['status'] );
  }

  /**
   * Checks that the API works with the exception flag.
   */
  public function testAPIReturnValueWithSpecialFlag() {
    if( IW::Env()->isNode() ) {
      $this->markTestSkipped( 'Not applicable on nodes' );
      return;
    }

    IW::Env()->setMaintenanceMode( 'unit-tests' );

    $url = 'http://127.0.0.1:2080/nodeworx/xmlrpc-api.php';

    $Client = new Zend_XmlRpc_Client( $url );
    $key    = array( 'email' => self::NW_USER, 'password' => self::NW_PASS );

    $params = array( 'apikey'    => $key,
                     'ctrl_name' => 'nodeworx/index',
                     'action'    => 'winCommit',
                     'input'     => array( 'run-in-maintenance-mode' => 1 ) );

    $result = $Client->call( 'iworx.route', $params );

    $this->assertEquals( API::SUCCESS, $result['status'] );
  }

  /**
   * Checks that the Old API will return the maintenance code.
   */
  public function testOldAPIReturnValue() {
    IW::Env()->setMaintenanceMode( 'unit-tests' );

    $url = 'http://127.0.0.1:2080/nodeworx/xmlrpc-api.php';

    $Client = new Zend_XmlRpc_Client( $url );
    $key    = array( 'email' => self::NW_USER, 'password' => self::NW_PASS );

    $params = array( 'apikey' => $key );

    $result = $Client->call( 'iworx.getip', $params );

    $this->assertEquals( API::ERROR_SERVER_MAINTENANCE, $result['status'] );
  }

  /**
   * Tests that maintenance mode returns a testable response code.
   */
  public function testWebHttpResponseCode() {
    IW::Env()->setMaintenanceMode( 'unit-tests' );

    $url = 'http://127.0.0.1:2080/nodeworx/index';

    $Client   = new Zend_Http_Client( $url );
    $Response = $Client->request();

    $this->assertEquals( 503, $Response->getStatus() );
  }

  /**
   * Tests that the XHR controller can respond without the 503.
   */
  public function testXHRHttpResponseCode() {
    IW::Env()->setMaintenanceMode( 'unit-tests' );

    $url = 'http://127.0.0.1:2080/xhr';

    $Client   = new Zend_Http_Client( $url );
    $Response = $Client->request();

    $this->assertEquals( 200, $Response->getStatus() );
  }

  /**
   * Tests the reply code of the XHR controller.
   */
  public function testXHRReplyCode() {
    IW::Env()->setMaintenanceMode( 'unit-tests' );

    $url = 'http://127.0.0.1:2080/xhr';

    $Client   = new Zend_Http_Client( $url );
    $Response = $Client->request();

    $json = json_decode( $Response->getBody() );

    $this->assertEquals( JSON_Reply::CODE_SERVER_MAINTENANCE, $json->c );
  }
}
