<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Jonathan Daniel <jdaniel@interworx.com>
 * @date    Apr 15, 2009
 */

/**
 * MySQL Driver Test.
 *
 * @package    InterWorx
 * @subpackage Tests
 *
 */
class MySQL_DriverTest extends IWorx_Framework_TestCase {

  const DB_NAME       = 'testdb';
  const USER_NAME     = 'testusr';
  const USER_PASSWORD = 'testing123';

  /**
   * Test to make sure MySQL is running.
   */
  public function testIsRunning() {
    $Driver = $this->_getDriver();

    $isRunning = $Driver->isRunning();
    $this->assertTrue( $isRunning );
  }

  /**
   * Test to make sure adding a database works.
   */
  public function testAddDatabase() {
    $Driver = $this->_getDriver();

    $this->_addDatabase( self::DB_NAME );

    $this->assertTrue( $Driver->databaseExists( self::DB_NAME ) == true );
  }

  /**
   * Test deleting the database.
   */
  public function testDeleteDatabase() {
    $Driver = $this->_getDriver();

    $this->_addDatabase( self::DB_NAME );
    $Driver->deleteDatabase( self::DB_NAME );
    $this->assertTrue( $Driver->databaseExists( self::DB_NAME ) == false );
  }

  /**
   * Test adding a user.
   */
  public function testAddUser() {
    $Driver = $this->_getDriver();

    $this->_addUser( self::USER_NAME, self::USER_PASSWORD );
    $this->assertTrue( $Driver->userExists( self::USER_NAME ) == true );
  }

  /**
   * Test deleting a user.
   */
  public function testDeleteUser() {
    $Driver = $this->_getDriver();

    $this->_addUser( self::USER_NAME, self::USER_PASSWORD );
    $Driver->deleteUser( self::USER_NAME );
    $this->assertTrue( $Driver->userExists( self::USER_NAME ) == false );
  }

  /**
   * Test the getAllowedVariables function.
   */
  public function testGetAllowedVariables() {
    $Driver = $this->_getDriver();

    $version = $Driver->getVersion();

    $params = $Driver->getAllowedVariables();
    if( $version > 4 ) {
      $this->assertTrue( empty( $params ) );
    } else {
      $this->assertTrue( !empty( $params ) );
    }
  }

  /**
   * Tests the start on boot functionality.
   */
  public function testStartOnBoot() {
    $Driver = $this->_getDriver();

    $oldSob = $Driver->isStartedOnBoot();

    $success = $Driver->startOnBoot( !$oldSob );
    $this->assertTrue( $success == true );

    $Driver->startOnBoot( $oldSob );
  }

  /**
   * TODO.
   * Make $Driver->getPermissions return an object/array that
   * $Driver->editPermissions can use as it's third argument.
   *
   * This should work.
   *
   * $Driver->editPermissions( $user,
   *                           $pass
   *                           $Driver->getPermissions( $user, $pass ) );
   */
  public function testEditPermissions() {
    $Driver = $this->_getDriver();

    $this->_addUser( self::USER_NAME, self::USER_PASSWORD );
    $this->_addDatabase( self::DB_NAME );

    $expected_permissions = $Driver->getPermList();
    $result = $Driver->editPermissions( self::USER_NAME,
                                        self::DB_NAME,
                                        $expected_permissions );

    $this->assertTrue( $result );
  }

  /**
   * Test for bug# 1130.
   */
  public function testEditFromGetPermissions() {
    $Driver = $this->_getDriver();

    $this->_addUser( self::USER_NAME, self::USER_PASSWORD );
    $this->_addDatabase( self::DB_NAME );

    $expected_permissions = $Driver->getPermList();
    $result = $Driver->editPermissions( self::USER_NAME,
                                        self::DB_NAME,
                                        $expected_permissions );

    $got_perms = $Driver->getPermissions( self::USER_NAME, self::DB_NAME );

    $result = $Driver->editPermissions( self::USER_NAME,
                                        self::DB_NAME,
                                        $got_perms );

    $this->assertTrue( $result );
  }

  /**
   * Tests the delete permissions.
   */
  public function testDeletePermissions() {
    $Driver = $this->_getDriver();

    $this->_addUser( self::USER_NAME, self::USER_PASSWORD );
    $this->_addDatabase( self::DB_NAME );

    $expected_permissions = $Driver->getPermList();
    $result = $Driver->editPermissions( self::USER_NAME,
                                        self::DB_NAME,
                                        $expected_permissions );

    $result = $result && $Driver->deletePermissions( self::USER_NAME,
                                                     self::DB_NAME );
    $this->assertTrue( $result );
  }

  /**
   * Tests the read config file function.
   *
   * TODO: Make this test not suck, but it gets us to the magical 1% where our
   * testing coverage is infinately better than before.
   */
  public function testReadConfigFile() {
    $Driver = $this->_getDriver();

    $msg = 'reading the config file failed, are you sure it exists?';

    $config = $Driver->readConfigFile();
    $this->assertTrue( is_array( $config ) && !empty( $config ), $msg );
  }

  /**
   * Tests the get users mapped to function.
   */
  public function testGetUsersMappedTo() {
    $Driver = $this->_getDriver();

    $this->_addUser( self::USER_NAME, self::USER_PASSWORD );
    $this->_addDatabase( self::DB_NAME );

    $perms = $Driver->getPermList();
    $Driver->editPermissions( self::USER_NAME,
                              self::DB_NAME,
                              $perms );

    $users = $Driver->getUsersMappedTo( self::DB_NAME );
    $found = false;
    foreach( $users as $user ) {
      if( $user->name == self::USER_NAME ) {
        $found = true;
        break;
      }
    }

    $this->assertTrue( $found === true );
  }

  /**
   * Tests that the restart function works properly.
   */
  public function testRestart() {
    $this->markTestSkipped( 'restarting mysql causing problems' );
    /*
      $Driver     = $this->_getDriver();
      $restart_ok = $Driver->restart( $block = true );
      $this->assertTrue( $restart_ok === true );
    */
  }

  /**
   * Tests that the magic PMA user works.
   *
   */
  public function testCreateMagicPMAUser() {
    $Driver = $this->_getDriver();

    $can_connect = true;
    $message     = null;

    try {
      $dsn = $Driver->createMagicPMAUser();
      IWorxDbUtil::db_connect( $dsn );
    } catch( IWorx_Exception_DbUtil $e ) {
      $can_connect = false;
      $message     = $e->getMessage();
    }

    $this->assertTrue( $can_connect, $message );
  }

  /**
   * Tests the canConnect driver function.
   */
  public function testCanConnect() {
    $Driver = $this->_getDriver();
    $this->assertTrue( $Driver->canConnect() );
  }

  /**
   * Tests the isLocal driver function.
   */
  public function testIsLocal() {
    $Driver   = $this->_getDriver();
    $DBServer = $Driver->getDatabaseServer();

    if( $DBServer->getIsLocal() == 'yes' ) {
      $this->assertTrue( $Driver->isLocal() === true );
    } else if( $DBServer->getIsLocal() == 'no' ) {
      $this->assertTrue( $Driver->isLocal() === false );
    } else {
      $this->assertTrue( false, 'Is local is neither yes or no' );
    }
  }

  /**
   * Tests the getLocalServer driver function.
   */
  public function testGetLocalServer() {
    $Driver   = $this->_getDriver();
    $DBServer = $Driver->getLocalServer();

    $this->assertTrue( $DBServer instanceof DatabaseServer );
  }

  /**
   * Gets the Database Driver.
   *
   * @return MySQL
   */
  private function _getDriver() {
    self::addSWAccountIfNecessary();

    $SiteWorx = new SiteWorx( self::SW_DOMAIN );
    $Driver   = $SiteWorx->getMySQLDriver();

    return $Driver;
  }

  /**
   * _addUser.
   *
   * @param string $username
   * @param string $password
   */
  private function _addUser( $username, $password ) {
    $Driver = $this->_getDriver();

    if( !$Driver->userExists( $username ) ) {
      $Driver->addUser( $username, $password );
    }
  }

  /**
   * _addDatabase.
   *
   * @param string $database
   */
  private function _addDatabase( $database ) {
    $Driver = $this->_getDriver();

    if( !$Driver->databaseExists( $database ) ) {
      $Driver->addDatabase( $database );
    }
  }
}
