<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Jonathan Daniel <jdaniel@interworx.com>
 * @date    Apr 15, 2009
 */

/**
 * MySQL_RoutineTest.
 *
 * @package    InterWorx
 * @subpackage Tests
 * @group      slow-tests
 */
class MySQL_RoutineTest extends IWorx_Framework_TestCase {

  const USER_NAME     = 'testusr';
  const USER_PASSWORD = 'testing123';

  const SECOND_USER_NAME     = 's2nd';
  const SECOND_USER_PASSWORD = 's2ndtest';

  const DB_NAME    = 'testdb';
  const TABLE_NAME = 'dummytable';

  const TEST_ROWS = 3;

  const ROUTINE_NAME = 'getCountRows';

  /**
   * Tests adding the routine.
   */
  public function testAddRoutine() {
    $this->_init( $init_routine = false );

    $ok = $this->_importTestRoutine();
    $this->assertTrue( $ok, $ok );
  }

  /**
   * Makes sure the routine's definer is what is expected.
   */
  public function testRoutineDefiner() {
    $this->_init();

    $ok = $this->_verifyRoutineDefiner( self::ROUTINE_NAME,
                                        self::USER_NAME );

    $this->assertTrue( $ok );
  }

  /**
   * Test the execution of the routine.
   */
  public function testRoutineExecution() {
    $this->_init();

    $SiteWorx = new SiteWorx( self::SW_DOMAIN );
    $Driver   = $SiteWorx->getMySQLDriver();

    $username = $Driver->getFQUN( self::USER_NAME );
    $password = self::USER_PASSWORD;
    $database = $Driver->getFQDN( self::DB_NAME );

    $dsn = $this->_getUserDsn( $username, $password, $database );

    $routine = self::ROUTINE_NAME;
    $sql     = "CALL {$routine}();";

    $result = IWorxDbUtil::db_runInCLI( $dsn, $sql, false );

    // We get the count of num_rows
    $this->assertTrue( $result[1] == self::TEST_ROWS );
  }

  /**
   * Test.
   *
   * Tests that a second test user will only have permissions to run the.
   * routine but not have any kind of permissions to execute SQL directly on
   * the database.
   */
  public function testRoutineWontRun() {
    $this->_init();

    $SiteWorx = new SiteWorx( self::SW_DOMAIN );
    $Driver   = $SiteWorx->getMySQLDriver();

    $username = $Driver->getFQUN( self::SECOND_USER_NAME );
    $password = self::SECOND_USER_NAME;
    $database = $Driver->getFQDN( self::DB_NAME );

    $dsn = $this->_getUserDsn( $username, $password, $database );
    $db  = IWorxDbUtil::db_connect( $dsn, $die = false );

    /*
     * This user should no be able to connect.
     */
    $this->assertTrue( Db::isError( $db ) === true );
  }

  /**
   * Now we give the secondary user permissions and let them run the routine.
   */
  public function testRunSecondaryUserRoutineWithPerms() {
    $this->_init();

    $SiteWorx = new SiteWorx( self::SW_DOMAIN );
    $Driver   = $SiteWorx->getMySQLDriver();

    $username = $Driver->getFQUN( self::SECOND_USER_NAME );
    $password = self::SECOND_USER_PASSWORD;
    $database = $Driver->getFQDN( self::DB_NAME );

    $dsn = $this->_getUserDsn( $username,
                               $password,
                               $database );

    $db = IWorxDbUtil::db_connect( $dsn, $die = false );

    $routine = self::ROUTINE_NAME;
    $sql     = "CALL {$routine}();";

    $result = IWorxDbUtil::db_runInCLI( $dsn, $sql, false );

    $this->assertTrue( $result[1] == self::TEST_ROWS );
  }

  /**
   * Test verify if routines still exist.
   */
  public function testVerifyRoutineStillExists() {
    $this->_init();

    $this->doBackupAndImport();
    $routine_exists = $this->_checkIfRoutineExists( self::DB_NAME,
                                                    self::ROUTINE_NAME );
    $this->assertTrue( $routine_exists == true );
  }

  /**
   * After a backup/import, make sure the definer remains the same.
   */
  public function testVerifyDefinerRemainsSame() {
    $this->_init();

    $this->doBackupAndImport();
    $definer_ok = $this->_verifyRoutineDefiner( self::ROUTINE_NAME,
                                                self::USER_NAME );
    $this->assertTrue( $definer_ok );
  }

  /**
   * Verify that the permissions remain after an import.
   */
  public function testVerifyPermsRemain() {
    $this->_init();

    $SiteWorx = new SiteWorx( self::SW_DOMAIN );
    $Driver   = $SiteWorx->getMySQLDriver();

    $this->doBackupAndImport();

    $username = $Driver->getFQUN( self::SECOND_USER_NAME );
    $password = self::SECOND_USER_PASSWORD;
    $database = $Driver->getFQDN( self::DB_NAME );

    $dsn = $this->_getUserDsn( $username, $password, $database );

    $db = IWorxDbUtil::db_connect( $dsn, $die = false );

    $routine = self::ROUTINE_NAME;
    $sql     = "CALL {$routine}();";

    $result = IWorxDbUtil::db_runInCLI( $dsn, $sql, false );

    $this->assertTrue( $result[1] == self::TEST_ROWS );

    /**
     * Here we are going to run a query on a table we don't have access to and.
     * expect an exception to be thrown
     */
    try {
      $table_name = self::TABLE_NAME;
      $sql        = "SELECT COUNT(*) FROM {$table_name};";

      $result = IWorxDBUtil::db_getOne( $db, $sql );

      $uname = self::SECOND_USER_NAME;

      $msg = "{$uname} had access to do something they shouldn't have been " .
        'able to do.';
      $this->assertTrue( true === false, $msg );
    } catch( IWorx_Exception_DbUtil $e ) {
      $this->assertTrue( true );
    }
  }

  /**
   * Test delete routines.
   */
  public function testDeleteRoutine() {
    $this->_init();

    $SiteWorx = new SiteWorx( self::SW_DOMAIN );
    $Driver   = $SiteWorx->getMySQLDriver();
    $db       = IWorxDbUtil::db_connect( $Driver->getDsn(), $die = false );

    $q_routine_name = q( self::ROUTINE_NAME );
    $sql = "DELETE
            FROM `proc`
            WHERE `name` = {$q_routine_name};";

    IWorxDbUtil::db_query( $db, $sql );

    $routine_exists = $this->_checkIfRoutineExists( self::DB_NAME,
                                                    self::ROUTINE_NAME );
    $this->assertTrue( $routine_exists == false );
  }

  /**
   * Init the unit test.
   *
   * @param boolean $init_routine
   */
  private function _init( $init_routine = true ) {
    self::deleteSWAccount();
    $ok = self::addSWAccount();
    $this->assertTrue( $ok );

    $SiteWorx = new SiteWorx( self::SW_DOMAIN );
    $Driver   = $SiteWorx->getMySQLDriver();

    /*
     * Add our test database
     */
    if( $Driver->databaseExists( self::DB_NAME ) === true ) {
      $Driver->deleteDatabase( self::DB_NAME );
    }
    $Driver->addDatabase( self::DB_NAME );

    /*
     * Add our test user
     */
    if( $Driver->userExists( self::USER_NAME ) === true ) {
      $Driver->deleteUser( self::USER_NAME );
    }
    $Driver->addUser( self::USER_NAME, self::USER_PASSWORD, '%' );

    /*
     * Give them permissions
     */
    $Driver->editPermissions( self::USER_NAME,
                              self::DB_NAME,
                              $Driver->getPermList(),
                              '%' );

    /*
     * Second test user
     */
    $uname = self::SECOND_USER_NAME;
    if( $Driver->userExists( $uname ) === true ) {
      // Wipe out any permissions the user has
      if( $Driver->permissionsExist( $uname, self::DB_NAME, '%' ) === true ) {
        $Driver->deletePermissions( $uname, self::DB_NAME, '%' );
      }
      $Driver->deleteUser( $uname );
    }
    $Driver->addUser( $uname, self::SECOND_USER_PASSWORD, '%' );

    /*
     * Second test user perms
     */
    $perms = array( 'EXECUTE' );
    $Driver->editPermissions( self::SECOND_USER_NAME,
                              self::DB_NAME,
                              $perms,
                              '%' );

    /*
     * Test Data
     */
    $this->_importTestData( self::TEST_ROWS );

    if( $init_routine === true ) {
      $this->_importTestRoutine();
    }
  }

  /**
   * Imports the test routine into a db with the username and password.
   *
   * @return boolean|string
   */
  private function _importTestRoutine() {
    $SiteWorx = new SiteWorx( self::SW_DOMAIN );
    $Driver   = $SiteWorx->getMySQLDriver();

    $database = $Driver->getFQDN( self::DB_NAME );

    /*
     * Delete the existing routine (if it exists) as the super user, so we
     * can be sure that an existing routine won't mess up the test.
     */
    $this->deleteRoutineAsSuper( $database, self::ROUTINE_NAME );

    $username = $Driver->getFQUN( self::USER_NAME );
    $password = self::USER_PASSWORD;
    $database = $Driver->getFQDN( self::DB_NAME );

    $dsn = $this->_getUserDsn( $username, $password, $database );

    $db = IWorxDbUtil::db_connect( $dsn );
    $r1 = IWorxDbUtil::db_query( $db, $this->_getDropRoutineSQL() );
    $r2 = IWorxDbUtil::db_query( $db, $this->_getRoutineSQL() );

    return ($r1 == 1 && $r2 == 1);
  }

  /**
   * Check to see if the given routine exists.
   *
   * @param string $database
   * @param string $routine_name
   * @return boolean
   */
  private function _checkIfRoutineExists( $database, $routine_name ) {
    $SiteWorx = new SiteWorx( self::SW_DOMAIN );
    $Driver   = $SiteWorx->getMySQLDriver();
    $db       = IWorxDbUtil::db_connect( $Driver->getDsn(), $die = false );

    $q_routine_name = q( $routine_name );
    $q_database     = q( $Driver->getFQDN( $database ) );

    $sql = "SELECT COUNT(*)
            FROM   `proc`
            WHERE  `name` = {$q_routine_name} AND
                   `db`   = {$q_database};";

    $routine_count = IWorxDbUtil::db_getOne( $db, $sql );
    return $routine_count > 0;
  }

  /**
   * Verify routine definer.
   *
   * @param string $routine_name
   * @param string $expected_definer
   * @param string $expected_host
   * @return boolean
   */
  private function _verifyRoutineDefiner( $routine_name,
                                          $expected_definer,
                                          $expected_host = '%' ) {

    $SiteWorx = new SiteWorx( self::SW_DOMAIN );
    $Driver   = $SiteWorx->getMySQLDriver();
    $db       = IWorxDbUtil::db_connect( $Driver->getDsn() );

    $expected_definer = $Driver->getFQUN( $expected_definer );
    $q_routine_name   = q( $routine_name );

    $sql  = 'SELECT `definer` ';
    $sql .= 'FROM `proc` ';
    $sql .= "WHERE `name` = {$q_routine_name} ";
    $sql .= "  AND `definer` LIKE \"$expected_definer@%\" ";
    $sql .= 'LIMIT 1';

    $definer = IWorxDbUtil::db_getOne( $db, $sql );
    $match   = preg_match( "/{$expected_definer}@{$expected_host}/", $definer );

    return $match >= 1;
  }

  /**
   * Imports test data into the database.
   *
   * @param integer $num_rows
   */
  private function _importTestData( $num_rows=1 ) {
    $SiteWorx = new SiteWorx( self::SW_DOMAIN );
    $Driver   = $SiteWorx->getMySQLDriver();

    $username = $Driver->getFQUN( self::USER_NAME );
    $password = self::USER_PASSWORD;
    $database = $Driver->getFQDN( self::DB_NAME );

    $dsn = $this->_getUserDsn( $username, $password, $database );
    $db  = IWorxDbUtil::db_connect( $dsn );

    IWorxDbUtil::db_query( $db, $this->_getDropTestDatabaseSql() );
    IWorxDbUtil::db_query( $db, $this->_getTestDatabaseSql() );

    for( $i = 0; $i < $num_rows; $i++ ) {
      $val = "RanData{$i}";
      IWorxDbUtil::db_query( $db, $this->_getInsertTestSql( $val ) );
    }
  }

  /**
   * Gets the dsn for the test database user.
   *
   * @param string $username
   * @param string $password
   * @param string $database
   * @return string
   */
  private function _getUserDsn( $username, $password, $database ) {
    $SiteWorx = new SiteWorx( self::SW_DOMAIN );
    $Driver   = $SiteWorx->getMySQLDriver();

    $pattern = '^mysql:\/\/(.+):(.+)@(.+)\/mysql$^';
    $replace = "mysql://{$username}:{$password}@$3/{$database}";

    $newdsn = preg_replace( $pattern, $replace, $Driver->getDsn() );

    return $newdsn;
  }

  /**
   * Runs the given sql command through the cli.
   *
   * @param string $dsn
   * @param string $sql
   * @return string
   */
  private function _runSQLinCLI( $dsn, $sql ) {
    $p        = Db::parseDSN( $dsn );
    $username = $p['username'];
    $database = $p['database'];
    $pass     = $p['password'];
    if( !empty( $p['hostspec'] ) ) {
      $hostOrSocket = "--host={$p['hostspec']}";
    } else {
      $hostOrSocket = "--socket={$p['socket']}";
    }

    $defaults = IWorxDbUtil::getTempDefaultsFile( $pass, IWorxDbUtil::USE_CLIENT );

    $tmp  = Ini::get( Ini::DIR, 'tmp' );
    $file = "{$tmp}/testsql.sql";

    $user  = Ini::get( Ini::IWORX, 'user' );
    $group = Ini::get( Ini::IWORX, 'group' );

    IWorxFileSys::smartBarf( $file, $sql, $user, $group );

    $cmd = Ini::get( Ini::BIN, 'mysql' ) .
      " --defaults-file={$defaults} " .
      " --user={$username} " .
      " {$hostOrSocket} " .
      " $database < $file ";

    $flags = IWorxExec::ALLOW_REDIR_IN | IWorxExec::STDERR_2_STDOUT;
    IWorxExec::exec( $cmd, $result, $retval, $flags );

    // Cleanup
    IWorxFileSys::rm( $file );
    IWorxFileSys::rm( $defaults );

    return $result;
  }

  /**
   * Adds a secondary test MySQL user.
   */
  private function _addSecondaryMySQLUser() {
    $SiteWorx = new SiteWorx( self::SW_DOMAIN );
    $Driver   = $SiteWorx->getMySQLDriver();
  }


  /**
   * Test drop test database.
   *
   * @return string
   */
  private function _getDropTestDatabaseSql() {
    $name = self::TABLE_NAME;

    $sql = "DROP TABLE IF EXISTS `{$name}`;";
    return $sql;
  }

  /**
   * Test table syntax.
   *
   * @return string
   */
  private function _getTestDatabaseSql() {
    $name = self::TABLE_NAME;

    $sql =
      "CREATE TABLE `{$name}` (
         `id` INT(11) AUTO_INCREMENT,
         `name` VARCHAR(30) NOT NULL,
         PRIMARY KEY ( `id` )
       ) ENGINE=MyISAM;";
    return $sql;
  }

  /**
   * Get insert sql.
   *
   * @param string $value
   * @return string
   */
  private function _getInsertTestSql( $value = 'abcdefg' ) {
    $name = self::TABLE_NAME;

    $q_value = q( $value );

    $sql =
      "INSERT INTO `{$name}` ( `name` )
       VALUES ( {$q_value} );";
    return $sql;
  }

  /**
   * Drop routine sql.
   *
   * @return string
   */
  private function _getDropRoutineSQL() {
    $name = self::ROUTINE_NAME;

    $sql = "DROP PROCEDURE IF EXISTS `{$name}`;";

    return $sql;
  }

  /**
   * Routines sql.
   *
   * @return string
   */
  private function _getRoutineSQL() {
    $routine_name = self::ROUTINE_NAME;
    $table_name   = self::TABLE_NAME;

    $sql = "CREATE PROCEDURE `{$routine_name}`()
       BEGIN
         SELECT COUNT(*) FROM {$table_name};
       END";

    return $sql;
  }
}
