<?php
/**
 * InterWorx Hosting Control Panel
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2017 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Upgrade
 * @author     Tim College <tcollege@interworx.com>
 * @date       Mar 22, 2017
 */

/**
 * Class Php_Scl70Test
 *
 * @package    InterWorx
 * @subpackage Test
 */
class Php_Scl70Test extends IWorx_Framework_TestCase {
  const TEST_DOMAIN      = 'iwtestphpscl70.com';
  const TEST_SCL_VERSION = '/opt/remi/php70';
  const TEST_UNIQNAME    = 'php70tst';

  /**
   * Setup.
   */
  static public function setUpBeforeClass() {
    self::deleteSWAccountIfNecessaryStatic( static::TEST_DOMAIN );
    if( PHP::isMultiplePhpInstalled() === false ) {
      PHP::enableMultiplePhp();
    }

    $Php = new PhpFpm( static::TEST_SCL_VERSION );
    if( $Php->isInstalled() === false ) {
      PHP::install( static::TEST_SCL_VERSION );
    }

    if( $Php->isEnabled() === false ) {
      PHP::setEnabledVersions( array_merge( PHP::getEnabledVersions(),
                                            [static::TEST_SCL_VERSION] ) );
    }

    self::_addSWAccount();
    self::_setUpWebroot();
  }

  /**
   * Loose verification of the setup function.
   */
  public function testSetupFiles() {
    IWorxLog::log( __CLASS__ . '::' . __METHOD__ );
    $SW = new SiteWorx( static::TEST_DOMAIN );

    $filename = $SW->getWebRoot() . '/php-mode-test/hello-world.php';
    $this->assertFileExists( $filename );
    $this->assertTrue( is_readable( $filename ) );
  }

  /**
   * Do a simple hello world test.
   */
  public function testPhpHelloWorldWorks() {
    IWorxLog::log( __CLASS__ . '::' . __METHOD__ );
    $this->_verifyHelloWorld();
  }

  /**
   * Verify that php-fpm is in use.
   */
  public function testPhpSapiName() {
    IWorxLog::log( __CLASS__ . '::' . __METHOD__ );
    $this->_verifyPhpSapi( 'fpm-fcgi' );
  }

  /**
   * Verify that we're running as the unix user of the account.
   */
  public function testWhoami() {
    IWorxLog::log( __CLASS__ . '::' . __METHOD__ );
    $this->_verifyExpectedUser( static::TEST_UNIQNAME );
  }

  /**
   * Verify that we're running the right version.
   */
  public function testVersion() {
    IWorxLog::log( __CLASS__ . '::' . __METHOD__ );
    $Php = new PhpFpm( static::TEST_SCL_VERSION );
    $this->_verifyVersion( $Php->getVersion() );
  }

  /**
   * Test backup and restore with the expectation it'll work.
   */
  public function testBackupRestoreWithCorrectVersionInstalled() {
    IWorxLog::log( __CLASS__ . '::' . __METHOD__ );

    $this->doBackupAndImport( null, static::TEST_DOMAIN );
    $SW  = new SiteWorx( static::TEST_DOMAIN );
    $Php = $SW->getPhp();

    $this->assertEquals( static::TEST_SCL_VERSION, $Php->getKey() );
    $this->_verifyPhpSapi( 'fpm-fcgi' );
    $this->_verifyExpectedUser( static::TEST_UNIQNAME );
    $this->_verifyVersion( $Php->getVersion() );
  }

  /**
   * Test what happens when the correct version is disabled.
   */
  public function testBackupRestoreWithCorrectVersionMissing() {
    IWorxLog::log( __CLASS__ . '::' . __METHOD__ );

    try {
      $this->doBackupAndImport( '_disablePhpVersion', static::TEST_DOMAIN );
    } catch( PHPUnit_Framework_AssertionFailedError $e ) {
      // We expect an error from the SiteWorx add failing.
      IworxLog::log( $e->getMessage() );
      return;
    }

    $this->fail( 'The SiteWorx add error did not get thrown, wtf?' );
  }

  /**
   * Disable the version under test.
   */
  protected function _disablePhpVersion() {
    $enabled = PHP::getEnabledVersions();
    $current = [ static::TEST_SCL_VERSION ];

    $new_enabled = array_diff( $enabled, $current );

    PHP::setEnabledVersions( $new_enabled );
  }

  /**
   * Add the test siteworx account.
   */
  private static function _addSWAccount() {
    IWorxLog::log( __CLASS__ . '::' . __METHOD__ );

    $input                                            = self::_getSWAccountInput();
    $input['master_domain']                           = static::TEST_DOMAIN;
    $input['uniqname']                                = static::TEST_UNIQNAME;
    $input[ Form_NW_Siteworx_Add::INPUT_PHP_VERSION ] = static::TEST_SCL_VERSION;

    self::addSWAccountFromInput( $input );
  }

  /**
   * Prepare the webroot with test files.
   */
  private static function _setUpWebroot() {
    IWorxLog::log( __CLASS__ . '::' . __METHOD__ );

    $SW = new SiteWorx( static::TEST_DOMAIN );
    IWorxFileSys::cp( __DIR__ . '/../../_resources/php-mode-test',
                      $SW->getWebRoot(),
                      true );

    IWorxFileSys::chown( $SW->getUniqname(),
                         $SW->getWebRoot(),
                         $SW->getUniqname(),
                         true );
    IWorxFileSys::chmod( 0711, $SW->getWebRoot() . '/php-mode-test' );
  }

  /**
   * Verify that hello world script works.
   */
  private function _verifyHelloWorld() {
    $response = $this->_getUrlContent( 'hello-world.php' );
    $this->assertEquals( 'Hello, world', trim( $response ) );
  }

  /**
   * Check the php sapi name.
   *
   * @param string $expected
   */
  private function _verifyPhpSapi( $expected ) {
    $actual = trim( $this->_getUrlContent( 'php-sapi-name.php' ) );
    $this->assertEquals( $expected, $actual );
  }

  /**
   * Verify that the script executes as the expected user.
   *
   * @param string $expected
   */
  private function _verifyExpectedUser( $expected ) {
    $actual = trim( $this->_getUrlContent( 'whoami.php' ) );
    $this->assertEquals( $expected, $actual );
  }

  /**
   * Verify that the version running is the version expected.
   *
   * @param string $expected
   */
  private function _verifyVersion( $expected ) {
    $actual = trim( $this->_getUrlContent( 'version.php' ) );
    $this->assertEquals( $expected, $actual );
  }

  /**
   * Gets the content served through apache.
   *
   * @param string $file
   * @return array
   */
  private function _getUrlContent( $file ) {
    $Domain   = new Domain( static::TEST_DOMAIN );
    $response = IWorxHTTP::httpGet( $Domain->getIPv4Address(),
                                    $Domain->getName(),
                                    "/php-mode-test/{$file}",
                                    false );
    return $response;
  }
}
