<?php
/**
 * InterWorx Hosting Control Panel
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2016 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Upgrade
 * @author     Tim College <tcollege@interworx.com>
 * @date       Jan 26, 2016
 */

/**
 * Class RpmUpgradeTest
 *
 * @package    InterWorx
 * @subpackage TESTS
 * @retryLimit 0
 */
class RpmUpgradeTest extends IWorx_Framework_TestCase {

  /**
   * Setup before class.
   *
   * This is needed for ioncube update bullshit.
   */
  static public function setupBeforeClass() {
    IW::Event();
    $I = new Input_String( 'x' );
    $I->validate();
    IWorxLog::log( 'Logging this line to load for ioncube' );
    $NW = new NodeWorx( NodeWorx::MASTER_ID );
    IW::FC()->routeFromPHP( 'Ctrl_Index', 'winCommit', [], $NW );
  }

  /**
   * Display the current interworx version installed.
   */
  protected function _displayCurrentIWorxVersion() {
    $Cmd = new Command( 'rpm -qi interworx' );
    $Cmd->exec();

    $this->_printCmdDetails( $Cmd );

    $Cmd = new Command( 'rpm -qi php-iworx' );
    $Cmd->exec();

    $this->_printCmdDetails( $Cmd );
  }

  /**
   * Verify that an upgrade is available to the interworx package.
   */
  public function testUpgradeAvailable() {
    $this->_displayCurrentIWorxVersion();
    $cmd  = Ini::get( Ini::BIN, 'yum' );
    $cmd .= ' info updates interworx';
    $Cmd  = new Command( $cmd );

    $Cmd->stderrToStdout();

    // On CentOS6, this will have retval 0 if update available, 1 otherwise
    $Cmd->exec();

    $this->_printCmdDetails( $Cmd );
    $this->assertEquals( CLI::OK, $Cmd->getRetval() );
  }

  /**
   * Run the daily cron.
   */
  public function testDailyCronRun() {
    $this->_displayCurrentIWorxVersion();
    $cmd  = Ini::get( Ini::IWORX_CRON, 'iworx' );
    $cmd .= ' --daily';

    $start_time = time();

    $Cmd = new Command( $cmd );
    $Cmd->exec();

    $this->_printCmdDetails( $Cmd );

    $end_time = time();

    $this->assertEquals( CLI::OK, $Cmd->getRetval() );
    // Verify that it took less than 15 minutes - this may need to be adjusted
    // In fact, it's pretty stupid, given that the daily launches yum in the
    // background now.
    $this->assertLessThanOrEqual( 900, $end_time - $start_time );
  }

  /**
   * Stall for time until the background upgrade appears complete.
   */
  public function testStallForTime() {
    $this->_displayCurrentIWorxVersion();
    // This env flag is added by Jenkins before the tests are run.
    $build = trim( IW::Env()->getEnvFlag( 'build-under-test' ) );
    $cmd   = Ini::get( Ini::BIN, 'rpm' );
    $cmd  .= ' -q interworx';

    $Cmd = new Command( $cmd );
    $Cmd->allowMultipleExecution();

    $start_time = time();
    $done       = false;

    /**
     * First, we have to stall until the RPM is present, which indicates that the
     * background task has started.
     *
     * We can NOT use the task itself - it sometimes completes too fast, and this
     * step would hang.
     */
    while( time() - $start_time < 900 && $done === false ) {
      $Cmd->exec();
      $this->_printCmdDetails( $Cmd );
      if( strpos( array_pop( $Cmd->getOutput() ), $build ) !== false ) {
        $done = true;
      } else {
        sleep( 3 );
      }
    }

    $this->assertTrue( $done, 'Timed out before upgrade completed' );

    $Cmd = new Command( 'ps auxf | grep run-one=YumUpdate | grep -v grep' );
    $Cmd->allowMultipleExecution();

    // Let it start
    sleep( 10 );

    /**
     * Now stall until the background task completes.
     *
     * It may already be complete - when there is little to do, it completes very
     * quickly.
     */
    while( $Cmd->exec() ) {
      $this->_printCmdDetails( $Cmd );
      sleep( 3 );
    }

    $this->_printCmdDetails( $Cmd );
  }

  /**
   * Verify that the upgrade has not left us in maintenance mode.
   */
  public function testNotInMaintenanceMode() {
    $count         = 0;
    $inMaintenance = true;

    /*
     * The rpm %post section seems to launch itself in the background, which means
     * that the maintenance mode files can be left in place until complete. We have
     * to attempt multiple times, because depending on what else was installed in
     * the transaction, other %post sections might execute first, and delay the time
     * until the maintenance mode files are cleaned up.
     */
    while( $count < 10 && $inMaintenance === true ) {
      $inMaintenance = IW::Env()->isInMaintenanceMode();
      ++$count;
      sleep( 10 );
    }
    $this->_displayCurrentIWorxVersion();
    passthru( 'ls -la ~iworx/etc/env/' );
    $this->assertFalse( $inMaintenance );
  }

  /**
   * Test that the upgrade is no longer available.
   *
   * This is a stupid way to go about it, but the actual build number isn't passed in
   * this far, so it wasn't feasable to make sure the "correct" build was installed.
   *
   * @depends testUpgradeAvailable
   */
  public function testUpgradeHappened() {
    $this->_displayCurrentIWorxVersion();
    $cmd  = Ini::get( Ini::BIN, 'yum' );
    $cmd .= ' info updates interworx';
    $Cmd  = new Command( $cmd );

    // On CentOS6, this will have retval 0 if update available, 1 otherwise
    $Cmd->exec();
    $this->_printCmdDetails( $Cmd );
    $this->assertNotEquals( CLI::OK, $Cmd->getRetval() );
  }

  /**
   * Verify that the correct build was installed.
   */
  public function testCorrectVersionInstalled() {
    $this->_displayCurrentIWorxVersion();
    // This env flag is added by Jenkins before the tests are run.
    $build = trim( IW::Env()->getEnvFlag( 'build-under-test' ) );
    $cmd   = Ini::get( Ini::BIN, 'rpm' );
    $cmd  .= ' -q interworx';

    $Cmd = new Command( $cmd );
    $Cmd->exec();

    $this->_printCmdDetails( $Cmd );

    $this->assertContains( $build, $Cmd->getOutput()[0] );
  }

  /**
   * Verifies that the repo was updated on upgrade.
   */
  public function testHotfixRepoUpdated() {
    $file  = CLI_Hotfix::HOTFIX_REPO;
    $build = trim( IW::Env()->getEnvFlag( 'build-under-test' ) );

    $this->assertTrue( is_file( $file ), 'Hotfix repo is missing' );
    $cmd = "grep baseurl /etc/yum.repos.d/interworx-hotfix.repo | grep /{$build}";

    $Cmd = new Command( $cmd );
    $Cmd->allowPipe();
    $Cmd->exec();

    $this->assertTrue( $Cmd->wasSuccessful() );
  }

  /**
   * Echo details of a command.
   *
   * @param Command $Cmd
   */
  protected function _printCmdDetails( Command $Cmd ) {
    IWorxConsole::echoLine( $Cmd->getCmd() );
    IWorxConsole::echoLine( 'Retval: ' . $Cmd->getRetval() );
    IWorxConsole::echoLine( $Cmd->getOutput() );
  }
}
