<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2011 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage TESTS
 * @author     Tim College <tcollege@interworx.com>
 * @date       Aug 29, 2011
 */

/**
 * Test for IPv6 Ranges.
 *
 * @package    InterWorx
 * @subpackage TESTS
 */
class IPv6_PoolTest extends IWorx_Framework_TestCase {
  const TEST_RANGE = '2001:db8::/96';
  const NEXT_IP    = '2001:db8::1';

  /**
   * Data provider.
   *
   * @return array
   */
  public function dataProviderInRange() {
    return
      array(
            array( '::1/120', '1', true ),
            array( '::1/120', '2', true ),
            array( '::1/120', IPv6::toDecimal( 'dead::beef' ), false ),
            array( '2001:db8:2::/96', IPv6::toDecimal( '2001:db8:2::2' ), true ),
            array( '2001:db8:2::/96', IPv6::toDecimal( '2001:db8:1::2' ), false ),
            array( '2001:db8:2::/96', IPv6::toDecimal( '2001:db8:3::2' ), false ),
            );
  }

  /**
   * Setup function.
   */
  static public function setUpBeforeClass() {
    $DbPool = IPv6_Pool::factoryFromDatabase( self::TEST_RANGE );

    if( $DbPool !== false ) {
      $DbPool->delete();
    }
  }

  /**
   * Tests the isInRange function.
   *
   * @param string  $range_cidr
   * @param string  $ip
   * @param boolean $expected
   * @dataProvider dataProviderInRange
   */
  public function testIsInRange( $range_cidr, $ip, $expected ) {
    $Pool = IPv6_Pool::factoryFromAddress( $range_cidr );

    $this->assertEquals( $expected, $Pool->isInRange( $ip ) );
  }

  /**
   * Tests range creation.
   */
  public function testCreateRange() {
    $Pool = IPv6_Pool::factoryFromAddress( self::TEST_RANGE );

    $Pool->setNickname( 'testrange' );
    $Pool->setDistributionPolicy( IPv6_Pool::POLICY_SEQUENTIAL );
    $Pool->setNetmaskCIDR( '128' );
    $Pool->setDevice( 'eth0' );

    $Pool->save();

    $DbPool = IPv6_Pool::factoryFromDatabase( self::TEST_RANGE );

    $this->assertInstanceOf( 'IPv6_Pool', $DbPool );
  }

  /**
   * Test getNextIp().
   */
  public function testGetNextIp() {
    $Pool = IPv6_Pool::factoryFromDatabase( self::TEST_RANGE );
    $next_ip = $Pool->getNextIp();

    $this->assertEquals( self::NEXT_IP, $next_ip );
  }

  /**
   * Test getIpObject().
   */
  public function testGetIpObject() {
    $Pool = IPv6_Pool::factoryFromDatabase( self::TEST_RANGE );
    $next_ip = $Pool->getNextIp();

    $IWorx_IPv6 = $Pool->getIpObject( $next_ip );

    $this->assertInstanceOf( 'IWorx_IPv6', $IWorx_IPv6 );
    $this->assertEquals( $next_ip, $IWorx_IPv6->getIp() );
    $this->assertEquals( '128', $IWorx_IPv6->getNetmask() );
    $this->assertEquals( 'eth0', $IWorx_IPv6->getDevice() );
    $this->assertTrue( $IWorx_IPv6->isStatic() );
    $this->assertTrue( $IWorx_IPv6->isVIP() );
    $this->assertFalse( $IWorx_IPv6->isReseller() );
  }

  /**
   * Test range deletion.
   *
   * @depends testCreateRange
   */
  public function testDeleteRange() {
    $Pool = IPv6_Pool::factoryFromDatabase( self::TEST_RANGE );
    $Pool->delete();

    $DbPool = IPv6_Pool::factoryFromDatabase( self::TEST_RANGE );
    $this->assertFalse( $DbPool, 'Deleting range failed.' );
  }

}
