<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Paul Oehler <poehler@interworx.com>
 * @date       Aug 12, 2008
 */

/**
 * Test class for IWorxFileSys.
 * Generated by PHPUnit on 2008-08-07 at 22:21:07.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class IWorxFileSysTest extends PHPUnit_Framework_TestCase {

  const EMPTY_FILE_MD5 = 'd41d8cd98f00b204e9800998ecf8427e';

  /**
   * Sample chmod codes for testing.
   *
   * @return array
   */
  public static function getChmodCodes() {
    return array( array( 0700 ),
                  array( 0600 ),
                  array( 0400 ),
                  array( 0777 ),
                  array( 0755 ),
                  array( 0644 ),
                  array( 04755 ),
                  array( 0000 )
                  );
  }

  /**
   * Returns a tmp directory for tesitng.
   *
   * @return string
   */
  public function getTempDir() {
    return Ini::get( Ini::DIR, 'tmp' );
  }

  /**
   * Returns a temporary file.
   *
   * @param string $indir
   * @return string
   */
  public function getTempFile( $indir = '' ) {
    if( $indir === '' ) {
      return tempnam( $this->getTempDir(), 'filesys_test_' );
    } else {
      return tempnam( $indir, 'filesys_test_' );
    }
  }

  /**
   * Test func.
   */
  public function testMd5sum1() {
    $result = IWorxFileSys::md5sum( $this->getTempFile() );
    $this->assertEquals( self::EMPTY_FILE_MD5,
                         $result,
                         'md5 returned wrong value for empty file' );
  }

  /**
   * Test func.
   */
  public function testMd5sum2() {
    $old = Ini::get( Ini::BIN, 'md5sum' );

    Ini::set( Ini::BIN, 'md5sum', '/does/not/exist/x' );

    $result = IWorxFileSys::md5sum( $this->getTempFile() );

    Ini::set( Ini::BIN, 'md5sum', $old );

    $this->assertEquals( self::EMPTY_FILE_MD5,
                         $result,
                         'md5_file returned wrong value for empty file' );
  }

  /**
   * Test func.
   */
  public function testChdir() {
    $cwd = getcwd();
    $tmp = $this->getTempDir();
    $this->assertNotEquals( $tmp, $cwd, 'Chdir src and dst must differ' );

    IWorxFileSys::chdir( $tmp );

    $newcwd = getcwd();
    $this->assertEquals( IWorxFileSys::realpath( $tmp ),
                         IWorxFileSys::realpath( $newcwd ) );
  }

  /**
   * Test func.
   */
  public function testChownOneFile() {
    $tmpfile = $this->getTempFile();

    IWorxFileSys::chown( 'vpopmail', $tmpfile );

    $fstat = stat( $tmpfile );
    unlink( $tmpfile );
    $this->assertEquals( 108, $fstat['uid'], 'Chown vpopmail user wrong uid' );
  }

  /**
   * Test func.
   */
  public function testChownAndChgrpOneFile() {
    $tmpfile = $this->getTempFile();

    IWorxFileSys::chown( 'vpopmail', $tmpfile, 'vchkpw' );

    $fstat = stat( $tmpfile );

    unlink( $tmpfile );

    $this->assertEquals( 108,
                         $fstat['uid'],
                         'Chown vpopmail wrong uid' );
    $this->assertEquals( 104,
                         $fstat['gid'],
                         'Chown vchkpw wrong gid' );
  }

  /**
   * Test func.
   */
  public function testChownRecursive() {
    $tmp       = $this->getTempDir();
    $newTmpDir = "$tmp/tmpdir";
    mkdir( $newTmpDir );
    $tmpfile = $this->getTempFile( $newTmpDir );

    IWorxFileSys::chown( 'vpopmail', $newTmpDir, 'vchkpw', $recurse = true );

    $dstat = stat( $newTmpDir );
    $fstat = stat( $tmpfile );

    unlink( $tmpfile );
    rmdir( $newTmpDir );

    $this->assertEquals( 108, $dstat['uid'], 'Chown dir vpopmail wrong uid' );
    $this->assertEquals( 104, $dstat['gid'], 'Chown dir vchkpw wrong gid' );
    $this->assertEquals( 108, $fstat['uid'], 'Chown file vpopmail wrong uid' );
    $this->assertEquals( 104, $fstat['gid'], 'Chown file vchkpw wrong gid' );
  }

  /**
   * Test func.
   */
  public function testChgrpOneFile() {
    $tmpfile = $this->getTempFile();

    IWorxFileSys::chgrp( 'vchkpw', $tmpfile );

    $fstat = stat( $tmpfile );
    unlink( $tmpfile );
    $this->assertEquals( 104, $fstat['gid'], 'Chgrp vchkpw wrong gid' );
  }

  /**
   * Test func.
   */
  public function testChgrpRecursive() {
    $tmp       = $this->getTempDir();
    $newTmpDir = "$tmp/tmpdir";
    mkdir( $newTmpDir );
    $tmpfile = $this->getTempFile( $newTmpDir );

    IWorxFileSys::chgrp( 'vchkpw', $newTmpDir, $recurse = true );

    $dstat = stat( $newTmpDir );
    $fstat = stat( $tmpfile );

    unlink( $tmpfile );
    rmdir( $newTmpDir );

    $this->assertEquals( 0,   $dstat['uid'], 'chgrp dir uid should be root' );
    $this->assertEquals( 104, $dstat['gid'], 'chgrp dir vchkpw wrong gid' );
    $this->assertEquals( 0,   $fstat['uid'], 'chgrp file, uid should be root' );
    $this->assertEquals( 104, $fstat['gid'], 'chgrp file vchkpw wrong gid' );
  }

  /**
   * Test func.
   *
   * @param integer $mode
   * @dataProvider getChmodCodes()
   */
  public function testChmodOneFile( $mode ) {
    $tmpfile = $this->getTempFile();

    IWorxFileSys::chmod( $mode, $tmpfile );

    $fperms = substr( sprintf( '%o', fileperms( $tmpfile ) ), -4 );
    unlink( $tmpfile );

    $smode = sprintf( '%o', $mode );
    $this->assertEquals( $smode, $fperms );
  }

  /**
   * Test func.
   *
   * @param integer $mode
   * @dataProvider getChmodCodes()
   */
  public function testChmodRecursive( $mode ) {
    $tmp       = $this->getTempDir();
    $newTmpDir = "$tmp/tmpdir";
    mkdir( $newTmpDir );
    $tmpfile = $this->getTempFile( $newTmpDir );

    IWorxFileSys::chmod( $mode, $newTmpDir, $recurse = true );

    $fperms = substr( sprintf( '%o', fileperms( $tmpfile ) ), -4 );
    $dperms = substr( sprintf( '%o', fileperms( $newTmpDir ) ), -4 );

    unlink( $tmpfile );
    rmdir( $newTmpDir );

    $smode = sprintf( '%o', $mode );
    $this->assertEquals( $smode, $dperms );
    $this->assertEquals( $smode, $fperms );
  }

  /**
   * Test func.
   */
  public function testChownmod() {
    $tmpfile = $this->getTempFile();
    $mode    = 0777;

    IWorxFileSys::chownmod( 'vpopmail', 'vchkpw', $mode, $tmpfile );

    $fperms = substr( sprintf( '%o', fileperms( $tmpfile ) ), -4 );
    $fstat  = stat( $tmpfile );
    unlink( $tmpfile );

    $smode = sprintf( '%o', $mode );
    $this->assertEquals( $smode, $fperms );
    $this->assertEquals( 108, $fstat['uid'], 'chown file vpopmail wrong uid' );
    $this->assertEquals( 104, $fstat['gid'], 'chgrp file vchkpw wrong gid' );
  }

  /**
   * Test func.
   */
  public function testMkdir() {
    $tmp = $this->getTempDir();

    IWorxFileSys::mkdir( "$tmp/mynewdir" );

    $direxists = is_dir( "$tmp/mynewdir" );
    rmdir( "$tmp/mynewdir" );
    $this->assertTrue( $direxists );
  }

  /**
   * Test func.
   */
  public function testMkdirWithSpaces() {
    $tmp        = $this->getTempDir();
    $dirname    = "$tmp/my newdir";

    IWorxFileSys::mkdir( $dirname, 0700, false );
    $this->assertTrue( is_dir( $dirname ) );

    rmdir( $dirname );
  }

  /**
   * Test func.
   */
  public function testMkdirSubdirWithSpaces() {
    $tmp        = $this->getTempDir();
    $dirname    = "$tmp/my newdir";
    $subdirname = $dirname . '/newerdir';

    IWorxFileSys::mkdir( $subdirname, 0700, true );
    $this->assertTrue( is_dir( $subdirname ) );

    rmdir( $subdirname );
    rmdir( $dirname );
  }

  /**
   * Test func.
   */
  public function testMkdirIworxWithSpaces() {
    $tmp        = $this->getTempDir();
    $dirname    = "$tmp/my newdir";

    IWorxFileSys::mkdir( $dirname, 0700, false, 'iworx', 'iworx' );
    $this->assertTrue( is_dir( $dirname ) );

    rmdir( $dirname );
  }

  /**
   * Test func.
   */
  public function testMkdirSubdirIworxWithSpaces() {
    $tmp        = $this->getTempDir();
    $dirname    = "$tmp/my newdir";
    $subdirname = $dirname . '/newerdir';

    IWorxFileSys::mkdir( $subdirname, 0700, true, 'iworx', 'iworx' );
    $this->assertTrue( is_dir( $subdirname ) );

    rmdir( $subdirname );
    rmdir( $dirname );
  }

  /**
   * Test func.
   */
  public function testMkdirWithMode() {
    $tmp  = $this->getTempDir();
    $mode = 0777;

    IWorxFileSys::mkdir( "$tmp/mynewdir", $mode );

    $direxists = is_dir( "$tmp/mynewdir" );
    $dperms    = substr( sprintf( '%o', fileperms( "$tmp/mynewdir" ) ), -4 );

    rmdir( "$tmp/mynewdir" );
    $this->assertTrue( $direxists );
    $smode = sprintf( '%o', $mode );
    $this->assertEquals( $smode, $dperms );
  }

  /**
   * Test func.
   */
  public function testMkdirWithModeRecursive() {
    $tmp  = $this->getTempDir();
    $mode = 0777;

    IWorxFileSys::mkdir( "$tmp/mynewdir/a/b/c/d/e", $mode, $recurse = true );

    $direxists = is_dir( "$tmp/mynewdir/a/b/c/d/e" );
    $dperms    = substr( sprintf( '%o',
                                  fileperms( "$tmp/mynewdir/a/b/c/d/e" ) ),
                         -4 );

    rmdir( "$tmp/mynewdir/a/b/c/d/e" );
    rmdir( "$tmp/mynewdir/a/b/c/d" );
    rmdir( "$tmp/mynewdir/a/b/c" );
    rmdir( "$tmp/mynewdir/a/b" );
    rmdir( "$tmp/mynewdir/a" );
    rmdir( "$tmp/mynewdir" );
    $this->assertTrue( $direxists );
    $smode = sprintf( '%o', $mode );
    $this->assertEquals( $smode, $dperms );
  }

  /**
   * Tests that the owner and group options work correctly.
   */
  public function testMkdirWithUser() {
    $tmp = $this->getTempDir();

    if( is_dir( "$tmp/mynewdir" ) === true ) {
      rmdir( "$tmp/mynewdir" );
    }

    IWorxFileSys::mkdir( "$tmp/mynewdir", 0700, $recurse = true );

    $stat    = IWorxFileSys::stat( "$tmp/mynewdir" );
    $pwnam   = posix_getpwuid( $stat['uid'] );
    $pwgrp   = posix_getgrgid( $stat['gid'] );

    $this->assertNotEquals( 'iworx', $pwnam['name'] );
    $this->assertNotEquals( 'iworx', $pwgrp['name'] );

    if( is_dir( "$tmp/mynewdir" ) === true ) {
      rmdir( "$tmp/mynewdir" );
    }

    IWorxFileSys::mkdir( "$tmp/mynewdir",
                         0700,
                         $recurse = true,
                         'iworx',
                         'iworx' );

    $stat    = IWorxFileSys::stat( "$tmp/mynewdir" );
    $pwnam   = posix_getpwuid( $stat['uid'] );
    $pwgrp   = posix_getgrgid( $stat['gid'] );

    rmdir( "$tmp/mynewdir" );

    $this->assertEquals( 'iworx', $pwnam['name'] );
    $this->assertEquals( 'iworx', $pwgrp['name'] );
  }
  /**
   * Test func.
   */
  public function testLn() {
    $tmpdir  = $this->getTempDir();
    $tmpfile = $this->getTempFile();

    IWorxFileSys::ln( $tmpfile, "$tmpdir/mylink" );
    $islink = is_link( "$tmpdir/mylink" );

    unlink( "$tmpdir/mylink" );
    unlink( $tmpfile );

    $this->assertTrue( $islink );
  }

  /**
   * Test func.
   */
  public function testMv() {
    $tmpdir  = $this->getTempDir();
    $tmpfile = $this->getTempFile();
    $ostat   = stat( $tmpfile );

    IWorxFileSys::mv( $tmpfile, "$tmpdir/movedfile" );

    $fstat = stat( "$tmpdir/movedfile" );
    unlink( "$tmpdir/movedfile" );

    $this->assertFalse( IWorxFileSys::isFile( $tmpfile ) );
    $this->assertEquals( $ostat['ino'], $fstat['ino'] );
  }

  /**
   * Test func.
   */
  public function testCp() {
    $tmpdir  = $this->getTempDir();
    $tmpfile = $this->getTempFile();
    $ostat   = stat( $tmpfile );

    IWorxFileSys::cp( $tmpfile, "$tmpdir/copiedfile" );

    $fstat = stat( "$tmpdir/copiedfile" );
    unlink( $tmpfile );
    unlink( "$tmpdir/copiedfile" );

    $this->assertNotEquals( $ostat['ino'], $fstat['ino'] );
  }

  /**
   * Test func.
   */
  public function testRm() {
    $tmpdir  = $this->getTempDir();
    $tmpfile = $this->getTempFile();

    IWorxFileSys::rm( $tmpfile );

    $this->assertFalse( IWorxFileSys::isFile( $tmpfile ) );
  }

  /**
   * Test rm when passed an array.
   */
  public function testArrayRm() {
    $tmparray = [$this->getTempFile(), $this->getTempFile()];
    IWorxFileSys::rm( $tmparray );
    foreach($tmparray as $tmpfile) {
      $this->assertFalse( IWorxFileSys::isFile( $tmpfile ) );
    }
  }

  /**
   * Spaces have caused issues at times, make sure there's no regression.
   */
  public function testRmWithSpaces() {
    $tmpdir = $this->getTempDir();
    $file   = $tmpdir . '/file with spaces';
    touch( $file );

    $this->assertTrue( file_exists( $file ) );

    IWorxFileSys::rm( $file );

    $this->assertFalse( file_exists( $file ) );
  }

  /**
   * Spaces have caused issues at times, make sure there's no regression.
   */
  public function testArrayRmWithSpaces() {
    $tmpdir = $this->getTempDir();
    $file   = $tmpdir . '/file with spaces';
    $file2   = $tmpdir . '/file with spaces again';
    $tmparray = [$file, $file2];

    touch( $file );
    touch( $file2 );

    $this->assertTrue( file_exists( $file ) );
    $this->assertTrue( file_exists( $file2 ) );

    IWorxFileSys::rm( $tmparray );

    $this->assertFalse( file_exists( $file ) );
    $this->assertFalse( file_exists( $file2 ) );
  }


  /**
   * Test func.
   */
  public function testTmpdir() {
    $tmpdir = $this->getTempDir();
    $mode   = 0777;

    $newDir = IWorxFileSys::tmpdir( $tmpdir, 'tmp_dir_test_', $mode );
    $dperms = substr( sprintf( '%o', fileperms( $newDir ) ), -4 );
    $is_dir = is_dir( $newDir );
    rmdir( $newDir );
    $smode = sprintf( '%o', $mode );
    $this->assertTrue( $is_dir );
    $this->assertEquals( $smode, $dperms );
  }

  /**
   * Test func.
   */
  public function testTempnam() {
    $tmpdir = $this->getTempDir();

    $tmpfile = IWorxFileSys::tempnam( IWorxFileSys::realpath( $tmpdir ),
                                      'tempnam_test_' );

    $is_file = IWorxFileSys::isFile( $tmpfile );
    if( $is_file ) {
      unlink( $tmpfile );
    }
    $this->assertTrue( $is_file );
  }

  /**
   * Test func.
   */
  public function testBackupFile() {
    $tmpfile = $this->getTempFile();

    IWorxFileSys::backupFile( $tmpfile );

    $is_file   = IWorxFileSys::isFile( $tmpfile );
    $dname     = dirname( $tmpfile );
    $fname     = basename( $tmpfile );
    $backup    = "$dname/.$fname";
    $is_file_b = IWorxFileSys::isFile( $backup );
    unlink( $tmpfile );
    unlink( $backup );
    $this->assertTrue( $is_file );
    $this->assertTrue( $is_file_b );
  }

  /**
   * Test func.
   */
  public function testRestoreFile() {
    $tmpfile = $this->getTempFile();

    IWorxFileSys::backupFile( $tmpfile );

    unlink( $tmpfile );

    $ret = IWorxFileSys::restoreFile( $tmpfile );

    $is_file = IWorxFileSys::isFile( $tmpfile );
    $dname   = dirname( $tmpfile );
    $fname   = basename( $tmpfile );
    $backup  = "$dname/.$fname";
    unlink( $tmpfile );
    unlink( $backup );

    $this->assertTrue( $is_file );
  }

  /**
   * Test func.
   */
  public function testSaveOrig() {
    $tmpfile = $this->getTempFile();

    IWorxFileSys::saveOrig( $tmpfile );

    $is_file   = IWorxFileSys::isFile( $tmpfile );
    $dname     = dirname( $tmpfile );
    $fname     = basename( $tmpfile );
    $backup    = "$dname/.$fname.orig";
    $is_file_b = IWorxFileSys::isFile( $backup );
    unlink( $tmpfile );
    unlink( $backup );
    $this->assertTrue( $is_file );
    $this->assertTrue( $is_file_b );
  }

  /**
   * Test func.
   */
  public function testLockBarf() {
    $tmpfile = $this->getTempFile();

    $r = IWorxFileSys::lockBarf( $tmpfile, 'new content',
                                 $b = false,
                                 $s = false );
    $this->assertTrue( $r );
    $contents = file_get_contents( $tmpfile );
    $this->assertEquals( 'new content', $contents, 'file contents written' );
    unlink( $tmpfile );
  }

  /**
   * Test func.
   */
  public function testSmartBarf() {
    $tmpfile = $this->getTempFile();

    $r = IWorxFileSys::smartBarf( $tmpfile,
                                  'new content',
                                  false,
                                  false,
                                  0600,
                                  false,
                                  false );

    $this->assertTrue( $r );
    $contents = file_get_contents( $tmpfile );
    $this->assertEquals( 'new content', $contents, 'file contents written' );

    unlink( $tmpfile );

    $r = IWorxFileSys::smartBarf( $tmpfile,
                                  'new content',
                                  false,
                                  false,
                                  0600,
                                  false,
                                  false );
    $this->assertTrue( $r );
    $contents = file_get_contents( $tmpfile );
    $this->assertEquals( 'new content', $contents, 'file contents written' );

    unlink( $tmpfile );
  }

  /**
   * Test func.
   */
  public function testFilesize() {
    $tmpfile = $this->getTempFile();
    file_put_contents( $tmpfile, 'new content' );
    $size = IWorxFileSys::filesize( $tmpfile );
    $this->assertEquals( 11, $size, 'correct file size' );
    unlink( $tmpfile );
  }

  /**
   * Test func.
   */
  public function testStat() {
    $tmpfile = $this->getTempFile();
    file_put_contents( $tmpfile, 'new content' );
    $stat = IWorxFileSys::stat( $tmpfile );
    $this->assertArrayHasKey( 'dev',   $stat, 'dev exists' );
    $this->assertArrayHasKey( 'ino',   $stat, 'ino exists' );
    $this->assertArrayHasKey( 'mode',  $stat, 'mode exists' );
    $this->assertArrayHasKey( 'nlink', $stat, 'nlink exists' );
    $this->assertArrayHasKey( 'uid',   $stat, 'uid exists' );
    $this->assertArrayHasKey( 'gid',   $stat, 'gid exists' );
    $this->assertArrayHasKey( 'rdev',  $stat, 'rdev exists' );
    $this->assertArrayHasKey( 'size',  $stat, 'size exists' );
    $this->assertArrayHasKey( 'atime', $stat, 'atime exists' );
    $this->assertArrayHasKey( 'mtime', $stat, 'mtime exists' );
    $this->assertArrayHasKey( 'ctime', $stat, 'ctime exists' );
    $this->assertArrayHasKey( 'blksize', $stat, 'blksize exists' );
    $this->assertArrayHasKey( 'blocks',  $stat, 'blocks exists' );
    unlink( $tmpfile );
  }

  /**
   * Test func.
   */
  public function testGetDirAvailableSpace() {
    $dir   = Ini::get( Ini::DIR, 'tmp' );
    $space = IWorxFileSys::getDirAvailableSpace( $dir );

    $this->assertTrue( is_numeric( $space ) );
  }

  /**
   * Test func.
   */
  public function testGetDirUsedSpace() {
    $tmpfile = $this->getTempFile();
    file_put_contents( $tmpfile, 'new content' );
    $r = IWorxFileSys::getDirUsedSpace( $this->getTempDir() );
    $this->assertTrue( is_numeric( $r ) );
    unlink( $tmpfile );
  }

  /**
   * Test func.
   */
  public function testGetMountmap() {
    $r = IWorxFileSys::getMountmap();
    $this->assertGreaterThan( 0, count( $r ) );
  }

  /**
   * Test func.
   */
  public function testEregGlob() {
    $r = IWorxFileSys::pregGlob( '/tmp', '/^.*$/' );
    $this->assertGreaterThanOrEqual( 2, count( $r ) );
  }

  /**
   * Test func.
   */
  public function testGetFstab() {
    $r = IWorxFileSys::getFstab();
    $this->assertGreaterThan( 0, count( $r ) );
  }

  /**
   * Test func.
   */
  public function testGetQuota() {
    $r = IWorxFileSys::getQuota( 'root' );
    $this->assertTrue( is_numeric( $r ) );
  }

  /**
   * Test func.
   */
  public function testGetRawQuotaData() {
    $r = IWorxFileSys::getRawQuotaData();
    $this->assertGreaterThan( 4, count( $r ) );
  }

  /**
   * Test func.
   */
  public function testSearchReplace() {
    $tempfile = $this->getTempFile();
    file_put_contents( $tempfile, 'you like it' );
    $r = IWorxFileSys::searchReplace( $tempfile, 'like', 'really like' );
    $this->assertTrue( $r );
    $content = file_get_contents( $tempfile );
    unlink( $tempfile );
    $this->assertEquals( 'you really like it', $content );
  }

  /**
   * Test func.
   */
  public function testDirectoryIsEmpty() {
    $dir = $this->getTempDir();
    @mkdir( "$dir/newdir" );
    $this->assertTrue( IWorxFileSys::directoryIsEmpty( "$dir/newdir" ) );
    $this->assertFalse( IWorxFileSys::directoryIsEmpty( $dir ) );
    rmdir( "$dir/newdir" );
  }

  /**
   * Test func.
   */
  public function testRemoveEmptyDirectoriesFrom() {
    $dir = $this->getTempDir();
    @mkdir( "$dir/newdir" );
    @mkdir( "$dir/newdir/boo" );
    $this->assertTrue( IWorxFileSys::removeEmptyDirectoriesFrom( "{$dir}/newdir" ) );
  }

  /**
   * Tests the IWorxFileSys::getIdenticalFiles function.
   */
  public function testGetIdenticalFiles() {
    $dir = $this->getTempDir();

    $dir1 = "$dir/testdir1";
    $dir2 = "$dir/testdir2";

    IWorxFileSys::mkdir( $dir1 );
    IWorxFileSys::mkdir( $dir2 );

    $content = 'This is a dummy file.';
    IWorxFileSys::smartBarf( "$dir1/dummyfile.txt", $content );
    IWorxFileSys::smartBarf( "$dir2/dummyfile.txt", $content );

    $files = IWorxFileSys::getIdenticalFiles( $dir1, $dir2 );
    $this->assertTrue( in_array( "$dir1/dummyfile.txt", $files ) );
    unset( $files );

    // now the bad file.
    IWorxFileSys::smartBarf( "$dir2/dummyfile2.txt", "$content and this is bad" );

    $files = IWorxFileSys::getIdenticalFiles( $dir1, $dir2 );

    IWorxFileSys::rm( $dir1, $r = true );
    IWorxFileSys::rm( $dir2, $r = true );

    $this->asserttrue( !empty( $files ) );
    $this->assertTrue( in_array( "$dir1/dummyfile.txt", $files ) );
    $this->assertTrue( !in_array( "$dir1/dummyfile2.txt", $files ) );
  }
}

