<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Tim College <tcollege@interworx.com>
 * @date    Apr 1, 2009
 */

/**
 * Tests that discover and run broad tests on all Input classes.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class Input_AllInputGenericTest extends IWorx_Framework_TestCase {
  /**
   * Gets an array of all the input classes.
   *
   * @return array
   */
  public function dataProviderInputClasses() {
    $files = IWorxFileSys::rglob( '*.php', 0, '/usr/local/interworx/include/Input' );

    $output = [];

    foreach( $files as $file ) {
      $parts = explode( '/', $file );
      array_shift( $parts ); // remove /
      array_shift( $parts ); // remove usr/
      array_shift( $parts ); // remove local/
      array_shift( $parts ); // remove interworx/
      array_shift( $parts ); // remove include/

      $class = implode( '_', $parts );
      $class = substr( $class, 0, -4 );

      if( $class === 'IWorx_Exception_Input' ) {
        continue;
      }

      if( $this->_isStandardConstructor( $class ) ) {
        $output[] = array( $class );
      }
    }

    //sort( $output );

    return $output;
  }

  /**
   * Gets an array of all the inputs and several data to test for each.
   *
   * @return array
   */
  public function dataProviderInputClassesAndData() {
    $output = [];

    foreach( $this->dataProviderInputClasses() as $array ) {
      $output[] = array( $array[0], 1 );
      $output[] = array( $array[0], '234' );
      $output[] = array( $array[0], 'a string' );
      $output[] = array( $array[0], 'email@email.com' );
      $output[] = array( $array[0], 'interworx.com' );
      $output[] = array( $array[0], array( '192.168.1.141' ) );
      $output[] = array( $array[0], array( 1, 2, 3 ) );
    }

    return $output;
  }

  /**
   * Gets an array of all the inputs and several bad data to test for each.
   *
   * @return array
   */
  public function dataProviderInputClassesAndBadData() {
    $output = [];

    foreach( $this->dataProviderInputClasses() as $array ) {
      $output[] = array( $array[0], "ti\tm is tricky" );
      $output[] = array( $array[0], "123\n" );
      $output[] = array( $array[0], 'a' . "\x1F" . 'string' );
      $output[] = array( $array[0], 'a' . "\x7F" . ' delete char' );
      $output[] = array( $array[0], array( "tabs\tare\tbad" ) );
    }

    return $output;
  }

  /**
   * Gets an array of all the inputs and several bad data to test for each.
   *
   * @return array
   */
  public function dataProviderInputClassesAndBadDataMult() {
    $output = [];

    foreach( $this->dataProviderInputClasses() as $array ) {
      $output[] = array( $array[0], "ti\tm is tricky" );
      $output[] = array( $array[0], "123\x11" );
      $output[] = array( $array[0], 'a' . "\x1F" . 'string' );
      $output[] = array( $array[0], 'a' . "\x7F" . ' delete char' );
      $output[] = array( $array[0], array( "tabs\tare\tbad" ) );
    }

    return $output;
  }

  /**
   * Uses reflection to check if it's a standard constructor.
   *
   * @param string $class
   * @return boolean
   */
  private function _isStandardConstructor( $class ) {
    $RC = new ReflectionClass( $class );

    if( $RC->isAbstract() ) {
      return false;
    }

    $Constructor = $RC->getConstructor();
    $Parameters  = $Constructor->getParameters();

    if( count( $Parameters ) < 3 ) {
      return false;
    }

    if( $Parameters[0]->getName() === 'name' &&
        $Parameters[1]->getName() === 'initial_value' &&
        $Parameters[2]->getName() === 'is_enabled' ) {
      return true;
    }

    return false;
  }

  /**
   * Gets an input.
   *
   * @param string  $class
   * @param string  $name
   * @param mixed   $initial_value
   * @param boolean $is_enabled
   * @return Input_Input
   */
  private function _getInput( $class,
                              $name = 'test',
                              $initial_value = null,
                              $is_enabled = true ) {
    return new $class( $name, $initial_value, $is_enabled );
  }

  /**
   * Some tests here require the dummy siteworx accounts.
   */
  public function testAddSwAccount() {
    self::addSWAccountIfNecessary();
  }

  /**
   * Tests the initial status of isRequired.
   *
   * @param string $class
   *
   * @dataProvider dataProviderInputClasses
   */
  public function testInputRequiredByDefault( $class ) {
    $Input = $this->_getInput( $class );
    if( $Input instanceof Input_Flag === false ) {
      $this->assertTrue( $Input->isRequired() );
    }
  }

  /**
   * Tests that the initial value is preserved.
   *
   * @param string $class
   *
   * @dataProvider dataProviderInputClasses
   */
  public function testInitialValue( $class ) {
    $Input = $this->_getInput( $class, 'name', 'initial value' );

    if( $Input->isMultipleAllowed() ) {
      $this->assertEquals( array( 'initial value' ), $Input->getInitialValue() );
    } else {
      $this->assertEquals( 'initial value', $Input->getInitialValue() );
    }
  }

  /**
   * Tests that the initial value is returned as current with no other interference.
   *
   * @param string $class
   *
   * @dataProvider dataProviderInputClasses
   */
  public function testCurrentValueIsInitialValue( $class ) {
    $Input = $this->_getInput( $class, 'name', 'initial value' );

    if( $Input->isMultipleAllowed() ) {
      $this->assertEquals( array( 'initial value' ), $Input->getCurrentValue() );
    } else {
      $this->assertEquals( 'initial value', $Input->getCurrentValue() );
    }
  }

  /**
   * Tests that the current value is returned.
   *
   * @param string $class
   *
   * @dataProvider dataProviderInputClasses
   */
  public function testCurrentValue( $class ) {
    $Input = $this->_getInput( $class, 'name', 'initial value' );

    if( $Input->isMultipleAllowed() ) {
      $Input->setCurrentValue( array( 'current value' ) );
      $this->assertEquals( array( 'current value' ), $Input->getCurrentValue() );
    } else {
      $Input->setCurrentValue( 'current value' );
      $this->assertEquals( 'current value', $Input->getCurrentValue() );
    }
  }

  /**
   * Tests that required inputs fail when given an empty string.
   *
   * @param string $class
   *
   * @dataProvider dataProviderInputClasses
   */
  public function testRequiredInputFailsWithEmptyString( $class ) {
    $Input = $this->_getInput( $class );

    $Input->setRequired( true );
    $Input->setCurrentValue( '' );

    $this->assertFalse( $Input->validate() );
  }

  /**
   * Tests that required inputs fail when given an empty array.
   *
   * @param string $class
   *
   * @dataProvider dataProviderInputClasses
   */
  public function testRequiredInputFailsWithEmptyArray( $class ) {
    $Input = $this->_getInput( $class );

    $Input->setRequired( true );
    $Input->setCurrentValue( array() );

    $this->assertFalse( $Input->validate() );
  }

  /**
   * Tests that required inputs fail when given an empty array when isMultiple.
   *
   * @param string $class
   *
   * @dataProvider dataProviderInputClasses
   */
  public function testRequiredInputFailsWithEmptyArrayAndMultiple( $class ) {
    $Input = $this->_getInput( $class );

    $Input->setRequired( true );
    $Input->setMultipleAllowed( true );
    $Input->setCurrentValue( array() );

    $this->assertFalse( $Input->validate() );
  }

  /**
   * Tests that required inputs fail when given an empty string.
   *
   * @param string $class
   *
   * @dataProvider dataProviderInputClasses
   */
  public function testNonRequiredInputPassesWithEmptyString( $class ) {
    $Input = $this->_getInput( $class );

    $Input->setCurrentValue( '' );
    $Input->setRequired( false );

    $this->assertTrue( $Input->validate() );
  }

  /**
   * Tests that required inputs fail when given an empty string and isMultiple.
   *
   * @param string $class
   *
   * @dataProvider dataProviderInputClasses
   */
  public function testNonRequiredInputPassesWithEmptyStringAndMultiple( $class ) {
    if( $class !== 'Input_Password' && $class !== 'Input_Password_Generate' ) {
      $Input = $this->_getInput( $class );

      // because isNewlineAllowed and multipleAllowed are mutually exclusive
      if( $Input instanceof Input_String ) {
        if( $Input->isNewlineAllowed() ) {
          return;
        }
      }

      $Input->setCurrentValue( '' );
      $Input->setRequired( false );
      $Input->setMultipleAllowed( true );

      $this->assertTrue( $Input->validate() );
    }
  }

  /**
   * Tests that the disabled copy is disabled.
   *
   * @param string $class
   *
   * @dataProvider dataProviderInputClasses
   */
  public function testGetDisabledCopyIsDisabled( $class ) {
    $Input = $this->_getInput( $class );

    $Copy = $Input->getDisabledCopy();

    $this->assertFalse( $Copy->isEnabled() );
  }

  /**
   * Tests that the disabled copy has a different name.
   *
   * @param string $class
   *
   * @dataProvider dataProviderInputClasses
   */
  public function testGetDisabledCopyNameChanges( $class ) {
    $Input = $this->_getInput( $class );

    $Copy = $Input->getDisabledCopy();

    $this->assertNotEquals( $Input->getName(), $Copy->getName() );
  }

  /**
   * Tests that the disabled copy initial value.
   *
   * @param string $class
   *
   * @dataProvider dataProviderInputClasses
   */
  public function testGetDisabledCopyInitialValue( $class ) {
    $Input = $this->_getInput( $class );
    $Input->setCurrentValue( 'Input Current Value' );

    $Copy = $Input->getDisabledCopy();

    $this->assertEquals( $Input->getCurrentValue(), $Copy->getInitialValue() );
  }

  /**
   * Tests that disabled inputs are immutable.
   *
   * @param string $class
   *
   * @dataProvider dataProviderInputClasses
   */
  public function testDisabledInputIsImmutableToUser( $class ) {
    IW::Env()->addInput( 'disabled', 'something else' );
    $Input = $this->_getInput( $class, 'disabled', 'initial', false );

    if( $Input->isMultipleAllowed() ) {
      $this->assertEquals( array( 'initial' ), $Input->getCurrentValue() );
    } else {
      $this->assertEquals( 'initial', $Input->getCurrentValue() );
    }
  }

  /**
   * Tests that disabled inputs are not immutable to the programmer.
   *
   * @param string $class
   *
   * @dataProvider dataProviderInputClasses
   */
  public function testDisabledInputIsNotImmutableToProgrammer( $class ) {
    $Input = $this->_getInput( $class, 'disabled', 'initial', false );

    $Input->setCurrentValue( 'current' );

    if( $Input->isMultipleAllowed() ) {
      $this->assertEquals( array( 'current' ), $Input->getCurrentValue() );
    } else {
      $this->assertEquals( 'current', $Input->getCurrentValue() );
    }
  }

  /**
   * Does some pretty broad testing - basically just runs the validators for luck.
   *
   * @param string  $class
   * @param mixed   $data
   * @param boolean $mult
   *
   * @dataProvider dataProviderInputClassesAndData
   */
  public function testValidationSomewhatRandomly( $class, $data, $mult = false ) {
    $Input = $this->_getInput( $class );
    $Input->setCurrentValue( $data );

    if( $mult === true ) {
      $Input->setMultipleAllowed( true );
    }

    $result = $Input->validate();

    if( $result === true ) {
      $this->assertEquals( array(), $Input->getErrors() );
    } else if( $result === false ) {
      $this->assertTrue( $Input->hasErrors() );
    } else {
      $this->fail( 'validate() did not return boolean!' );
    }
  }

  /**
   * Does some pretty broad testing with multiple enabled.
   *
   * @param string $class
   * @param mixed  $data
   *
   * @dataProvider dataProviderInputClassesAndData
   */
  public function testValidationSomewhatRandomlyWithMultiple( $class, $data ) {
    $this->testValidationSomewhatRandomly( $class, $data, true );
  }

  /**
   * Tests low-ascii chars.
   *
   * @param string  $class
   * @param string  $data
   * @param boolean $mult
   *
   * @dataProvider dataProviderInputClassesAndBadData
   */
  public function testValidationWithBadData( $class, $data, $mult = false ) {
    if( $class === 'Input_Html' ) {
      // HTML Purifier filters the bad shit, not validates
      return;
    }

    $Input = $this->_getInput( $class );

    if( $mult === false &&
        $Input instanceof Input_String &&
        $Input->isNewlineAllowed() ) {
      return;
    }

    $Input->setCurrentValue( $data );

    if( $mult === true ) {
      $Input->setMultipleAllowed( true );
    }

    $this->assertFalse( $Input->validate() );
  }

  /**
   * Tests low-ascii chars.
   *
   * @param string  $class
   * @param string  $data
   *
   * @dataProvider dataProviderInputClassesAndBadDataMult
   */
  public function testValidationWithBadDataMult( $class, $data ) {
    $this->testValidationWithBadData( $class, $data, true );
  }
}
