<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Tim College <tcollege@interworx.com>
 * @date    Mar 11, 2009
 */

/**
 * Tests for Input_SetCustom.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class Input_SetCustomTest extends IWorx_Framework_TestCase {
  const TEST_INPUT_NAME   = 'test_input';
  const CUSTOM_INPUT_NAME = 'custom_input';

  /**
   * Builds an input for testing.
   *
   * @param string|null  $initial_value
   * @param integer|null $custom_input_value
   * @return Input_SetCustom
   */
  private function _getInput( $initial_value = null,
                              $custom_input_value = null ) {
    $CustomInput = new Input_Integer( self::CUSTOM_INPUT_NAME, $custom_input_value );
    $DS          = new DataSource_Array( range( 'a', 'f' ) );
    $Input       = new Input_SetCustom( self::TEST_INPUT_NAME,
                                        $DS,
                                        $CustomInput,
                                        $initial_value );
    return $Input;
  }

  /**
   * Tests that getCustomInput returns an input object.
   */
  public function testGetCustomInputReturnsInput() {
    $Input = $this->_getInput();

    $this->assertInstanceOf( 'Input_Input', $Input->getCustomInput() );
  }

  /**
   * Tests that setting the value to an option in the datasource works.
   */
  public function testSettingInitialValueWorksWithChoiceInDataSource() {
    $Input = $this->_getInput( 'c' );

    $this->assertEquals( 'c', $Input->getInitialValue() );
  }

  /**
   * Tests that the initial value of the custom input is returned as initial of input.
   */
  public function testSettingInitialValueOfCustomInputReturnsAsRealValue() {
    $Input = $this->_getInput( null, 24 );

    $this->assertEquals( 24, $Input->getInitialValue() );
  }

  /**
   * Tests that setting the value to an option in the datasource works.
   */
  public function testSettingCurrentValueWorksWithChoiceInDataSource() {
    $Input = $this->_getInput( 'c' );

    $this->assertEquals( 'c', $Input->getCurrentValue() );
  }

  /**
   * Tests that the initial value of the custom input is returned as initial of input.
   */
  public function testSettingCurrentValueOfCustomInputReturnsAsRealValue() {
    $Input = $this->_getInput( null, 24 );

    $this->assertEquals( 24, $Input->getCurrentValue() );
  }

  /**
   * Tests that environment input works from the actual input.
   */
  public function testSettingEnvironmentInputForRealInput() {
    IW::Env()->addInput( self::TEST_INPUT_NAME, 'e' );

    $Input = $this->_getInput();

    $this->assertEquals( 'e', $Input->getCurrentValue() );
  }

  /**
   * Tests that the enrironment input works from the custom input.
   */
  public function testSettingEnvironmentInputForCustomInput() {
    IW::Env()->addInput( self::CUSTOM_INPUT_NAME, 18 );

    $Input = $this->_getInput();

    $this->assertEquals( 18, $Input->getCurrentValue() );
  }

  /**
   * Tests that the REAL initial value is returned if the real current is not set.
   */
  public function testSettingCustomEnvironmentInputReturnsRealInitialValue() {
    IW::Env()->addInput( self::CUSTOM_INPUT_NAME, 18 );

    $Input = $this->_getInput( 'e' );

    $this->assertEquals( 'e', $Input->getCurrentValue() );
  }

  /**
   * Tests that the custom input value is returned when the real input is special.
   */
  public function testCustomInputReturnedWhenRealInputIsSpecial() {
    IW::Env()->addInput( self::TEST_INPUT_NAME, Input_SetCustom::CUSTOM_VALUE );
    IW::Env()->addInput( self::CUSTOM_INPUT_NAME, 18 );

    $Input = $this->_getInput( 'e' );

    $this->assertEquals( 18, $Input->getCurrentValue() );
  }

  /**
   * Tests validators.
   */
  public function testThatDataSourceOptionsAreValid() {
    IW::Env()->addInput( self::TEST_INPUT_NAME, 'a' );

    $Input = $this->_getInput( 'e' );

    $this->assertEquals( 'a', $Input->getCurrentValue() );
    $this->assertTrue( $Input->validate(), 'Input did not validate' );
  }

  /**
   * Tests validators.
   */
  public function testIntegersNotInTheDataSourceAreValid() {
    IW::Env()->addInput( self::TEST_INPUT_NAME, Input_SetCustom::CUSTOM_VALUE );
    IW::Env()->addInput( self::CUSTOM_INPUT_NAME, 18 );

    $Input = $this->_getInput( 'e' );

    $this->assertEquals( 18, $Input->getCurrentValue() );
    $this->assertTrue( $Input->validate(), 'Input did not validate' );
  }

  /**
   * Tests validators.
   */
  public function testNonIntegersAreInvalid() {
    IW::Env()->addInput( self::TEST_INPUT_NAME, Input_SetCustom::CUSTOM_VALUE );
    IW::Env()->addInput( self::CUSTOM_INPUT_NAME, 'boogers' );

    $Input = $this->_getInput( 'e' );

    $this->assertEquals( 'boogers', $Input->getCurrentValue() );
    $this->assertFalse( $Input->validate(), 'Input did not invalidate bullshit' );
  }

  /**
   * Tests that the initial value is passed to the custom input if appropriate.
   */
  public function testGetCurrentValueOfCustomInput() {
    $Input = $this->_getInput( 16 );

    $CustomInput = $Input->getCustomInput();

    $this->assertEquals( 16, $CustomInput->getCurrentValue(),
                         'Current value of the custom input is incorrect' );
  }

  /**
   * Tests that the current value of the custom input is null.
   *
   * If the current value of the actual input is a valid datasource value
   */
  public function testGetCurrentValueOfCustomInputWithDataSourceInitialValue() {
    $Input = $this->_getInput( 'a' );

    $CustomInput = $Input->getCustomInput();

    $this->assertNull( $CustomInput->getCurrentValue(),
                       'Current value of the custom input is incorrect' );
  }

  /**
   * Tests manually setting current value.
   */
  public function testSetCurrentValueWithNonDataSourceValueSetsCustomValue() {
    $Input = $this->_getInput( 'a' );

    $Input->setCurrentValue( 25 );

    $CustomInput = $Input->getCustomInput();

    $this->assertEquals( 25, $CustomInput->getCurrentValue() );
  }

  /**
   * Tests the setting multiple allowed fails.
   *
   * @expectedException IWorx_Exception
   */
  public function testSetMultipleAllowedThrowsException() {
    $Input = $this->_getInput( 'a' );
    $Input->setMultipleAllowed( true );
  }
}
