<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Tim College <tcollege@interworx.com>
 * @date    Oct 31, 2008
 */

/**
 * Test Case for Input_String.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class Input_StringTest extends PHPUnit_Framework_TestCase {
  const INPUT_NAME = 'test_input';

  /**
   * Gets an input.
   *
   * @param string $initial_value
   * @return Input_String
   */
  private function _getInput( $initial_value = '' ) {
    return new Input_String( self::INPUT_NAME, $initial_value );
  }

  /**
   * Make sure the defaults aren't stupid.
   */
  public function testDefaultValues() {
    $I = $this->_getInput();

    // 0 is ok, because of non-required inputs
    $this->assertEquals( 0, $I->getMinLength(), 'Minimum length is stupid!' );
    $this->assertGreaterThan( $I->getMinLength(), $I->getMaxLength(),
                              'Max length was less than min length' );
  }

  /**
   * Tests the behavior of get and set MinLength.
   */
  public function testMinLength() {
    $I = $this->_getInput();

    $this->assertEquals( 0, $I->getMinLength(), 'Unexpected default value' );

    $I->setLengthRange( 12 );
    $this->assertEquals( 12, $I->getMinLength(),
                         'Min length is not what I set!' );

    $I->setLengthRange( 1000 );
    $this->assertTrue( $I->hasErrors(),
                       'Setting min > max did not create an error!' );

    // Get a new input
    $I = $this->_getInput();
    $this->assertFalse( $I->hasErrors() );
    $I->setLengthRange( -12 );
    $this->assertTrue( $I->hasErrors() );
    $this->assertEquals( 0, $I->getMinLength() );

    // Get a new input
    $I = $this->_getInput();
    $this->assertFalse( $I->hasErrors() );
    $I->setLengthRange( -12 );
    $this->assertTrue( $I->hasErrors() );
    $this->assertEquals( 0, $I->getMinLength() );

    // Get a new input
    $I = $this->_getInput();
    $this->assertFalse( $I->hasErrors() );
    $I->setLengthRange( 1000, 2500 );
    $this->assertFalse( $I->hasErrors() );
  }

  /**
   * Tests the bahavior of getMaxLength and setMaxLength.
   */
  public function testMaxLength() {
    $I = $this->_getInput();

    $this->assertGreaterThan( 0, $I->getMaxLength(),
                              'Default maxlength is dumb' );

    $I->setLengthRange( 1, 30 );

    $this->assertEquals( 30, $I->getMaxLength(), 'Max length is not what I set' );

    $I->setLengthRange( 1, -1 );

    $this->assertTrue( $I->hasErrors(), 'Setting max < 0 did not create an error' );

    //get a new input
    $I = $this->_getInput();

    // Try to set max less than min
    $I->setLengthRange( 10, 9 );

    $this->assertTrue( $I->hasErrors(), 'Setting max < min did not create error' );

    $this->assertGreaterThan( $I->getMinLength(), $I->getMaxLength(),
                              'Max length is less than min length' );
  }

  /**
   * Tests behavior with valid input.
   */
  public function testValidatePassesWithValidInput() {
    $I = $this->_getInput();

    $I->setCurrentValue( 'A medium longish string.' );

    $this->assertTrue( $I->validate(), 'Valid input did not validate' );
    $this->assertFalse( $I->hasErrors(), 'Input has errors' );
  }

  /**
   * Tests behavior with input that's too long.
   */
  public function testValidateFailsWithLongInput() {
    // Test input that's too long
    $I = $this->_getInput();
    $I->setLengthRange( 1, 5 );
    $I->setCurrentValue( 'A medium longish string.' );

    $this->assertFalse( $I->validate(), 'Invalid input did not fail' );
    $this->assertTrue( $I->hasErrors(), 'Input does not have errors' );
  }

  /**
   * Tests behavior with input that's too short.
   */
  public function testValidateFailsWithShortInput() {
    // Test input that's not long enough
    $I = $this->_getInput();
    $I->setLengthRange( 100 );
    $I->setCurrentValue( 'A medium longish string.' );

    $this->assertFalse( $I->validate(), 'Invalid input did not fail' );
    $this->assertTrue( $I->hasErrors(), 'Input does not have errors' );
  }

  /**
   * Tests the behavior of a bug with empty strings and multiple inputs.
   */
  public function testMultipleEmptyInputBug() {
    $I = $this->_getInput();
    $I->setMultipleAllowed( true );

    $this->assertEquals( array(), $I->getCurrentValue() );
  }

  /**
   * Verifies that the initial value has not changed.
   */
  public function testInitialValueUnchangedWithFilter() {
    $I = $this->_getInput( '"This has double-quotes"' );
    $I->addFilter( new Zend_Filter_StringTrim( '"' ) );

    $this->assertEquals( '"This has double-quotes"',
                         $I->getInitialValue() );
  }

  /**
   * Test that getUnfilteredCurrentValue returns the unfiltered value.
   */
  public function testFilterGetUnfilteredCurrentValue() {
    $I = $this->_getInput( '"This has double-quotes"' );
    $I->addFilter( new Zend_Filter_StringTrim( '"' ) );

    $this->assertEquals( '"This has double-quotes"',
                         $I->getUnfilteredCurrentValue() );
  }

  /**
   * Test that getCurrentValue returns the filtered value.
   */
  public function testFilterGetCurrentValue() {
    $I = $this->_getInput( '"This has double-quotes"' );
    $I->addFilter( new Zend_Filter_StringTrim( '"' ) );

    $this->assertEquals( 'This has double-quotes', $I->getCurrentValue() );
  }

  /**
   * Test that getCurrentValue returns the filtered value for an array.
   */
  public function testFilterGetCurrentValueWithArray() {
    $I = $this->_getInput( '"This has double-quotes"' );
    $I->addFilter( new Zend_Filter_StringTrim( '"' ) );
    $I->setMultipleAllowed( true );

    $this->assertEquals( array( 'This has double-quotes' ), $I->getCurrentValue() );
  }

  /**
   * Tests that newlines still work after fixing bug 1507.
   */
  public function testNewLinesGetPastControlCharValidator() {
    $I = $this->_getInput( "One\nTwo" );
    $I->setNewlineAllowed( true );

    $this->assertTrue( $I->validate() );
    $this->assertEquals( "One\nTwo", $I->getCurrentValue() );
  }
}
