<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Jonathan Daniel <jdaniel@interworx.com>
 * @date    Nov 12, 2010
 */

require_once 'DummyModels.php';

/**
 * Tests the Model functionality.
 *
 * @package    IntertWorx
 * @subpackage Tests
 */
class Model_AbstractTest extends IWorx_Framework_TestCase {

  /**
   * Tests that create returns a blank instance of the object.
   */
  public function testCreate() {
    $Object = Model_Test_Record::create();

    $this->assertTrue( $Object instanceof Model_Test_Record );
    $this->assertFalse( $Object->exists() );
  }

  /**
   * Tests that getting the primary key works.
   */
  public function testGetPrimaryKey() {
    $Object = Model_Test_Record::create();
    $Other  = Model_Other_Test_Record::create();

    $this->assertEquals( 'id', $Object->getPrimaryKey() );
    $this->assertEquals( 'other_id', $Other->getPrimaryKey() );
  }

  /**
   * Tests that create populates the object properly.
   */
  public function testCreateWithData() {
    $create_data = array( 'id' => 20, 'field1' => 'hello world' );
    $Object      = Model_Test_Record::create( $create_data );

    $this->assertEquals( 20, $Object->id );
    $this->assertEquals( 'hello world', $Object->field1 );
  }

  /**
   * Tests that the id is set correctly after insert.
   */
  public function testSettingIdAfterInsert() {
    $Orm = new Model_Test_Record();

    $Orm->field1       = 'test';
    $Orm->null_field   = null;
    $Orm->field_two    = 'test';
    $Orm->field_3_here = 'test';
    $Orm->timestamp    = IW::now();
    $Orm->some_int     = 5;
    $Orm->some_enum    = 'val1';

    $Orm->save();

    $this->assertEquals( 1, $Orm->getPrimaryId() );
  }

  /**
   * Tests that load will work correctly.
   */
  public function testLoadWorksCorrectly() {
    $Model = Model_Test_Record::load( 1 );

    $this->assertEquals( $Model->getPrimaryId(), 1 );
    $this->assertTrue( $Model instanceof Model_Test_Record );
    $this->assertEquals( 'test', $Model->field1 );
  }

  /**
   * Tests that null get loaded properly.
   */
  public function testNullValueLoadsCorrectly() {
    $Model = Model_Test_Record::load( 1 );

    $this->assertEquals( null, $Model->null_field );
  }

  /**
   * Tests that load will throw an exception with an invalid id.
   */
  public function testLoadThrowsExceptionWithInvalidId() {
    $e = null;
    try {
      $Model = Model_Test_Record::load( 'herpderp' );
    } catch( Exception $e ) {
      //eat it.
    }
    $this->assertTrue( $e instanceof IWorx_Exception_Model_InvalidRecord );
  }

  /**
   * Tests that load will throw an exception with a nonestant id.
   */
  public function testLoadThrowsExceptionWithNonexistantId() {
    $e = null;
    try {
      $Model = Model_Test_Record::load( 99992 );
    } catch( Exception $e ) {
      //eat it.
    }
    $this->assertTrue( $e instanceof IWorx_Exception_Model_InvalidRecord );
  }

  /**
   * Tests that load doesn't work from within the base class.
   */
  public function testLoadWontWorkFromBaseClass() {
    $e = null;
    try {
      $Model = Model_Abstract::load( 1 );
    } catch( Exception $e ) {
      //eat it.
    }
    $this->assertTrue( $e instanceof IWorx_Exception_Model_InvalidClass );
  }

  /**
   * Tests that save throws an exception with invalid data.
   */
  public function testSaveThrowsExceptionWithInvalidData() {
    $Orm               = new Model_Test_Record();
    $Orm->field1       = 'test';
    $Orm->null_field   = null;
    $Orm->field_two    = null;
    $Orm->field_3_here = null;
    $Orm->timestamp    = null;
    $Orm->some_int     = null;
    $Orm->some_enum    = null;

    // Should throw an exception here because there are null fields.

    try {
      $Orm->save();
      $this->assertTrue( false );
    } catch( Exception $e ) {
      $this->assertTrue( $e instanceof IWorx_Exception_Model_InvalidData );
    }
  }

  /**
   * Tests that trying to set an invalid property throws an exception.
   */
  public function testThatInvalidPropertyThrowsException() {
    $Model  = new Model_Test_Record();
    $caught = false;

    try {
      $Model->this_is_invalid = 'blah';
    } catch( IWorx_Exception_Model_InvalidProperty $e ) {
      $caught = true;
    }

    $this->assertTrue( $caught );
  }

  /**
   * Tests setting a Forgien Key.
   *
   * This method seems a little half-assed... We should come up with something
   * better.
   */
  public function testSettingForgienKey() {
    $Other  = Model_Other_Test_Record::create();
    $Record = Model_Test_Record::load( 1 );

    $Other->test_record_id = $Record;

    $this->assertEquals( $Record->getPrimaryId(), $Other->test_record_id );
  }

  /**
   * Tests loading an existing record.
   */
  public function testLoadingAnExistingRecord() {
    $Orm = new Model_Test_Record( 1 );

    $this->assertEquals( 1, $Orm->getPrimaryId() );
  }

  /**
   * Model Test.
   *
   * Tests that saving an existing record updates the current record and does
   * not insert a new one.
   */
  public function testSavingExistingRecordUpdatesAndNotInserts() {
    $Orm = new Model_Test_Record( 1 );

    $Orm->save();
    $this->assertEquals( 1, $Orm->getPrimaryId() );
  }

  /**
   * Tests setting properties.
   */
  public function testSettingProperties() {
    $Orm = new Model_Test_Record( 1 );

    $Orm->field1 = 'hello world';
    $Orm->save();

    $this->assertEquals( 'hello world', $Orm->field1 );
  }

  /**
   * Tests that enabling read only mode won't allow changes.
   */
  public function testReadOnlyWontAllowChanges() {
    $Orm       = new Model_Test_Record( 1 );
    $new_field = 'this should not save!';

    $Orm->enableReadOnly();

    $Orm->field1 = 'THIS SHOULD NOT SAVE!!!!!!!!!';
    $Orm->save();

    $this->assertNotEquals( $new_field, $Orm->field1 );
  }

  /**
   * Tests that saving a datetime works correctly.
   */
  public function testSavingADatetime() {
    $now_in_unixtime = IW::now();
    $Model_Test = new Model_Test_Record( 1 );
    $Model_Test->timestamp = $now_in_unixtime;

    $Model_Test->save();

    // Now we load a second time, the timestamp should return a unix datetime.
    $Model_Test = new Model_Test_Record( 1 );

    $this->assertEquals( $now_in_unixtime, $Model_Test->timestamp, 'not equal' );
  }

  /**
   * Tests saving the value as an enum type.
   */
  public function testSavingAnEnum() {
    $Model_Test = new Model_Test_Record( 1 );
    $Model_Test->some_enum = 'val2';

    $Model_Test->save();

    $Model_Test = new Model_Test_Record( 1 );

    $this->assertEquals( 'val2', $Model_Test->some_enum, 'not equal' );
  }

  /**
   * Tests that find works with an id.
   */
  public function testFindWithId() {
    $Orm = Model_Test_Record::find( 1 );

    $this->assertEquals( 1, $Orm->getPrimaryId() );
  }

  /**
   * Tests that $Object->exists() works properly.
   */
  public function testExists() {
    $Orm = Model_Test_Record::find( 1 );

    $this->assertTrue( $Orm->exists() );
  }

  /**
   * Tests that find works with an array.
   */
  public function testFindWithArray() {
    $tests = Model_Test_Record::find( array( 'field1' => 'hello world' ) );

    $this->assertTrue( is_array( $tests ) );
    $this->assertEquals( 1, count( $tests ) );
  }

  /**
   * Tests the delete does not work in read only mode.
   */
  public function testDeleteWontWorkInReadonly() {
    $Orm = Model_Test_Record::load( 1 );

    $Orm->enableReadOnly();

    $ok      = $Orm->delete();
    $New_Orm = Model_Test_Record::load( 1 );

    $this->assertFalse( $ok );
    $this->assertEquals( 1, $New_Orm->getPrimaryId() );
  }

  /**
   * Tests delete.
   */
  public function testDelete() {
    $Orm     = Model_Test_Record::load( 1 );
    $success = $Orm->delete();

    $this->assertTrue( $success );
  }

  /**
   * Tests that the record was actually deleted.
   */
  public function testRecordWasActuallyDeleted() {
    $e = null;

    try {
      $NewOrm = Model_Test_Record::load( 1 );
    } catch( Exception $e ) {
      // eat it.
    }

    $this->assertTrue( $e instanceof IWorx_Exception_Model_InvalidRecord );
  }

  /**
   * Tests that trying to delete an already deleted record fails gracefully.
   */
  public function testDeletingAlreadyDeleteRecordFailsGracefully() {
    $Orm     = Model_Test_Record::create( array( 'id' => 1 ) );
    $success = $Orm->delete();

    $this->assertTrue( $success );
  }

  /**
   * Tests that find returns all records when no value is given.
   */
  public function testFindWithNoValues() {
    $Orm = new Model_Test_Record();

    $Orm->field1       = 'test';
    $Orm->null_field   = null;
    $Orm->field_two    = 'test';
    $Orm->field_3_here = 'test';
    $Orm->timestamp    = IW::now();
    $Orm->some_int     = 5;
    $Orm->some_enum    = 'val1';

    $Orm->save();

    $objects = Model_Test_Record::find();

    $this->assertEquals( 1, count( $objects ) );
  }

  /**
   * Deletes all current test data.
   */
  private function _deleteAllCurrentData() {
    self::setUpBeforeClass();
  }

  /**
   * Loads a bunch of test data.
   */
  private function _loadTestData() {
    for( $i = 0; $i < 10; $i++ ) {
      $Orm = Model_Test_Record::create();

      $Orm->field1       = "test{$i}";
      $Orm->field_two    = "test2{$i}";
      $Orm->field_3_here = "test3{$i}";

      $Orm->save();
    }
  }

  /**
   * Tests that the find limit works correctly.
   */
  public function testFindLimit() {
    $this->_deleteAllCurrentData();
    $this->_loadTestData();

    $objects = Model_Test_Record::find( null, '5, 10' );

    $this->assertEquals( 5, count( $objects ) );
  }

  /**
   * Stup before the class is executed.
   */
  static public function setUpBeforeClass() {
    Model_Test_Record::buildTable();
    Model_Other_Test_Record::buildTable();
  }

  /**
   * Tear down after the class is executed.
   */
  static public function tearDownAfterClass() {
    Model_Test_Record::killTable();
    Model_Other_Test_Record::killTable();
  }
}