<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Tim College <tcollege@interworx.com>
 * @date       Nov 3, 2010
 */

/**
 * Tests related to the Payload object.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class PayloadTest extends IWorx_Framework_TestCase {
  /**
   * Checks the number of rows in a string of html representing a table.
   *
   * @param integer $expected
   * @param string  $html
   */
  public function assertTableRowCount( $expected, $html ) {
    $actual = $this->_getRowCount( $html );

    $this->assertEquals( $expected, $actual, 'Row count did not match' );
  }

  /**
   * Checks the number of columns in a string of html representing a table.
   *
   * @param integer $expected
   * @param string  $html
   */
  public function assertTableColumnCount( $expected, $html ) {
    $actual = $this->_getColumnCount( $html );

    $this->assertEquals( $expected, $actual, 'Column count did not match' );
  }

  /**
   * Gets the number of rows in the html.
   *
   * @param string $html
   * @return integer
   */
  private function _getRowCount( $html ) {
    $matches = [];
    return preg_match_all( '#<tr.+?</tr>#s', $html, $matches );
  }

  /**
   * Gets the count of columns per row.
   *
   * @param string $html
   * @return integer
   */
  private function _getColumnCount( $html ) {
    $matches = [];
    $cell_count = preg_match_all( '#<td.+?</td>#s', $html, $matches );

    return $cell_count / $this->_getRowCount( $html );
  }

  /**
   * Data provider of various values for testing single-value payloads.
   *
   * @return array
   */
  public function dataProviderSingleValues() {
    return array( array( 'a string' ),
                  array( 23456 ),
                  array( 0 ),
                  array( '0' ),
                  array( '14' ),
                  array( true ),
                  array( false ),
                  array( 2.341 ),
    );
  }

  /**
   * Data to assert that what goes in is what comes out.
   *
   * @return array
   */
  public function dataProviderValues() {
    $reply_data = 'The data in the reply';
    $Reply      = new Reply( Reply::OK_WITH_DATA, $reply_data );

    $values = $this->dataProviderSingleValues();

    $values[] = array( range( 0, 12 ) );
    $values[] = array( $Reply, $reply_data );

    $data = [];

    $object = (object) null;
    $object->field1 = 'whatever';
    $object->field2 = 'something else';
    $object->field3 = 'the last field';

    $data[] = $object;
    $data[] = $object;
    $data[] = $object;

    $values[] = $data;

    return $values;
  }

  /**
   * Verifies that the output of getData() is as expected.
   *
   * @param mixed $input
   * @param mixed $output
   * @dataProvider dataProviderValues
   */
  public function testPayloadGetData( $input, $output = null ) {
    if( $output === null ) {
      $output = $input;
    }

    $Payload = new Payload( $input );

    $this->assertEquals( $output, $Payload->getData() );
  }

  /**
   * Checks that it has no actions by default.
   */
  public function testHasActions1() {
    $Payload = new Payload( 'whatever' );

    $this->assertFalse( $Payload->hasActions() );
  }

  /**
   * Checks that it has actions after adding one.
   */
  public function testHasActions2() {
    $Payload = new Payload( 'whatever' );

    $Payload->addAction( 'Action', '##LG_DELETE##' );

    $this->assertTrue( $Payload->hasActions() );
  }

  /**
   * Tests column definition default behaviour.
   */
  public function testThatDefaultColumnsAreBuiltAutomagically() {
    $array = [];
    $array['field1'] = 'whatever';
    $array['field2'] = 'something else';
    $array['field3'] = 'the last field';

    $Payload = new Payload( $array );

    $columns = $Payload->getColumns();

    $this->assertCount( 3, $columns );

    foreach( $columns as $Column ) {
      $this->assertInstanceOf( 'Payload_Column', $Column );
    }
  }

  /**
   * Tests single values.
   *
   * @param mixed $input
   * @dataProvider dataProviderSingleValues
   */
  public function testBuildPayloadSingleValue( $input ) {
    $output = Payload_Renderer::buildPayload( new Payload( $input ) );

    $this->assertTableRowCount( 2, $output );
    $this->assertTableColumnCount( 1, $output );
    $this->assertContains( IWorxUtil::stringify( $input ), $output );
  }

  /**
   * Tests building payload with a single-dimensional numeric array.
   */
  public function testBuildPayloadNumericArray() {
    $array = [];
    $array[] = 'whatever';
    $array[] = 'something else';
    $array[] = 'the last field';

    $output = Payload_Renderer::buildPayload( new Payload( $array ) );

    $this->assertTableRowCount( 2, $output );
    $this->assertTableColumnCount( 3, $output );

    foreach( $array as $item ) {
      $this->assertContains( $item, $output );
    }
  }

  /**
   * Tests building payload with a single-dimensional associative array.
   */
  public function testBuildPayloadAssocArray() {
    $array = [];
    $array['field1'] = 'whatever';
    $array['field2'] = 'something else';
    $array['field3'] = 'the last field';

    $output = Payload_Renderer::buildPayload( new Payload( $array ) );

    $this->assertTableRowCount( 2, $output );
    $this->assertTableColumnCount( 3, $output );

    foreach( $array as $item ) {
      $this->assertContains( $item, $output );
    }
  }

  /**
   * Tests building payload with an object.
   */
  public function testBuildPayloadObject() {
    $object = (object) null;
    $object->field1 = 'whatever';
    $object->field2 = 'something else';
    $object->field3 = 'the last field';

    $output = Payload_Renderer::buildPayload( new Payload( $object ) );

    $this->assertTableRowCount( 2, $output );
    $this->assertTableColumnCount( 3, $output );

    foreach( $object as $item ) {
      $this->assertContains( $item, $output );
    }
  }

  /**
   * Tests building a payload.
   */
  public function testBuildPayloadArrayOfObjects() {
    $data = [];

    $object = (object) null;
    $object->field1 = 'whatever';
    $object->field2 = 'something else';
    $object->field3 = 'the last field';

    $data[] = $object;
    $data[] = $object;
    $data[] = $object;

    $output = Payload_Renderer::buildPayload( new Payload( $data ) );

    $this->assertTableRowCount( 4, $output );
    $this->assertTableColumnCount( 3, $output );

    // Assert that some of the data is actually in the output
    $this->assertContains( 'whatever', $output );

    // Assert that the lang key of the header was guessed
    $this->assertContains( '##LG_FIELD1##', $output );
  }

  /**
   * Tests building a payload.
   */
  public function testBuildPayloadArrayOfNumericArrays() {
    $data = [];

    $array = [];
    $array[] = 'whatever';
    $array[] = 'something else';
    $array[] = 'the last field';

    $data[] = $array;
    $data[] = $array;
    $data[] = $array;

    $output = Payload_Renderer::buildPayload( new Payload( $data ) );

    $this->assertTableRowCount( 4, $output );
    $this->assertTableColumnCount( 3, $output );

    // Assert that some of the data is actually in the output
    $this->assertContains( 'whatever', $output );

    $this->assertNotContains( '##LG_0##', $output, 'Header lang key made from a ' .
                                                   'number, that\'s pretty dumb' );
  }

  /**
   * Tests building a payload.
   */
  public function testBuildPayloadArrayOfAssociativeArrays() {
    $data = [];

    $array = [];
    $array['field1'] = 'whatever';
    $array['field2'] = 'something else';
    $array['field3'] = 'the last field';

    $data[] = $array;
    $data[] = $array;
    $data[] = $array;

    $output = Payload_Renderer::buildPayload( new Payload( $data ) );

    $this->assertTableRowCount( 4, $output );
    $this->assertTableColumnCount( 3, $output );

    // Assert that some of the data is actually in the output
    $this->assertContains( 'whatever', $output );

    // Assert that the lang key of the header was guessed
    $this->assertContains( '##LG_FIELD1##', $output );
  }

  /**
   * Tests building a payload.
   */
  public function testBuildPayloadColumns() {
    $data = [];

    $object = (object) null;
    $object->field1 = 'whatever';
    $object->field2 = 'something else';
    $object->field3 = 'the last field';

    $data[] = $object;
    $data[] = $object;
    $data[] = $object;

    $Payload = new Payload( $data );

    $Payload->setColumn( new Payload_Column( 'field1') );

    $output = Payload_Renderer::buildPayload( $Payload );

    $this->assertTableRowCount( 4, $output );
    $this->assertTableColumnCount( 1, $output );

    // Assert that some of the data is actually in the output
    $this->assertContains( 'whatever', $output );

    $this->assertNotContains( $object->field2, $output );

    // Assert that the lang key of the header was guessed
    $this->assertContains( '##LG_FIELD1##', $output );
  }

  /**
   * Payloads that use column definitions explicitly must have certain data structs.
   *
   * @expectedException IWorx_Exception
   */
  public function testBuildPayloadWrong() {
    $Payload = new Payload( 'string' );

    $Payload->setColumn( new Payload_Column( 'kaboom' ) );
  }
}
